@@ -35,7 +35,7 @@ import org.apache.tools.ant.BuildException;
  * the Tomcat manager application.
  *
  * @author Craig R. McClanahan
- * @version $Id: DeployTask.java 939336 2010-04-29 15:00:41Z kkolinko $
+ * @version $Id: DeployTask.java 939305 2010-04-29 13:43:39Z kkolinko $
  * @since 4.1
  */
 public class DeployTask extends AbstractCatalinaTask {
@@ -137,6 +137,7 @@ public class DeployTask extends AbstractCatalinaTask {
      *
      * @exception BuildException if an error occurs
      */
+    @Override
     public void execute() throws BuildException {
 
         super.execute();
@@ -185,7 +186,7 @@ public class DeployTask extends AbstractCatalinaTask {
         }
 
         // Building URL
-        StringBuffer sb = new StringBuffer("/deploy?path=");
+        StringBuilder sb = new StringBuilder("/deploy?path=");
         try {
             sb.append(URLEncoder.encode(this.path, getCharset()));
             if ((war == null) && (config != null)) {
