@@ -29,13 +29,15 @@ import java.security.PrivilegedActionException;
 import java.security.PrivilegedExceptionAction;
 import java.text.SimpleDateFormat;
 import java.util.ArrayList;
+import java.util.Collection;
 import java.util.Enumeration;
-import java.util.Iterator;
+import java.util.List;
 import java.util.Locale;
 import java.util.TimeZone;
 import java.util.Vector;
 
 import javax.servlet.ServletOutputStream;
+import javax.servlet.SessionTrackingMode;
 import javax.servlet.http.Cookie;
 import javax.servlet.http.HttpServletResponse;
 
@@ -43,23 +45,24 @@ import org.apache.catalina.Context;
 import org.apache.catalina.Globals;
 import org.apache.catalina.Session;
 import org.apache.catalina.Wrapper;
+import org.apache.catalina.core.ApplicationSessionCookieConfig;
 import org.apache.catalina.security.SecurityUtil;
 import org.apache.catalina.util.CharsetMapper;
 import org.apache.catalina.util.DateTool;
-import org.apache.catalina.util.StringManager;
 import org.apache.tomcat.util.buf.CharChunk;
 import org.apache.tomcat.util.buf.UEncoder;
 import org.apache.tomcat.util.http.FastHttpDateFormat;
 import org.apache.tomcat.util.http.MimeHeaders;
 import org.apache.tomcat.util.http.ServerCookie;
 import org.apache.tomcat.util.net.URL;
+import org.apache.tomcat.util.res.StringManager;
 
 /**
  * Wrapper object for the Coyote response.
  *
  * @author Remy Maucherat
  * @author Craig R. McClanahan
- * @version $Id: Response.java 944398 2010-05-14 19:00:21Z markt $
+ * @version $Id: Response.java 1088458 2011-04-04 00:57:17Z kkolinko $
  */
 
 public class Response
@@ -68,9 +71,21 @@ public class Response
 
     // ----------------------------------------------------------- Constructors
 
+    /**
+     * Compliance with SRV.15.2.22.1. A call to Response.getWriter() if no
+     * character encoding has been specified will result in subsequent calls to
+     * Response.getCharacterEncoding() returning ISO-8859-1 and the Content-Type
+     * response header will include a charset=ISO-8859-1 component.
+     */
+    private static final boolean ENFORCE_ENCODING_IN_GET_WRITER;
+
     static {
         // Ensure that URL is loaded for SM
         URL.isSchemeChar('c');
+
+        ENFORCE_ENCODING_IN_GET_WRITER = Boolean.valueOf(
+                System.getProperty("org.apache.catalina.connector.Response.ENFORCE_ENCODING_IN_GET_WRITER",
+                        "true")).booleanValue();
     }
 
     public Response() {
@@ -85,13 +100,13 @@ public class Response
      * Descriptive information about this Response implementation.
      */
     protected static final String info =
-        "org.apache.coyote.tomcat5.CoyoteResponse/1.0";
+        "org.apache.coyote.catalina.CoyoteResponse/1.0";
 
 
     /**
      * The string manager for this package.
      */
-    protected static StringManager sm =
+    protected static final StringManager sm =
         StringManager.getManager(Constants.Package);
 
 
@@ -220,12 +235,6 @@ public class Response
     protected boolean error = false;
 
 
-    /**
-     * The set of Cookies associated with this Response.
-     */
-    protected ArrayList<Cookie> cookies = new ArrayList<Cookie>();
-
-
     /**
      * Using output stream flag.
      */
@@ -267,8 +276,6 @@ public class Response
         error = false;
         isCharacterEncodingSet = false;
         
-        cookies.clear();
-
         if (Globals.IS_SECURITY_ENABLED || Connector.RECYCLE_FACADES) {
             if (facade != null) {
                 facade.clear();
@@ -301,17 +308,28 @@ public class Response
 
 
     /**
-     * Return the number of bytes actually written to the output stream.
+     * Return the number of bytes the application has actually written to the
+     * output stream. This excludes chunking, compression, etc. as well as
+     * headers.
      */
-    public int getContentCount() {
+    public long getContentWritten() {
         return outputBuffer.getContentWritten();
     }
-    
+
+
     /**
-     * Return the number of bytes actually written to the output stream.
+     * Return the number of bytes the actually written to the socket. This
+     * includes chunking, compression, etc. but excludes headers.
      */
-    public long getContentCountLong() {
-        return outputBuffer.getContentWrittenLong();
+    public long getBytesWritten(boolean flush) {
+        if (flush) {
+            try {
+                outputBuffer.flush();
+            } catch (IOException ioe) {
+                // Ignore - the client has probably closed the connection
+            }
+        }
+        return coyoteResponse.getBytesWritten(flush);
     }
 
     /**
@@ -330,7 +348,7 @@ public class Response
     public boolean isAppCommitted() {
         return (this.appCommitted || isCommitted() || isSuspended()
                 || ((getContentLength() > 0) 
-                    && (getContentCount() >= getContentLength())));
+                    && (getContentWritten() >= getContentLength())));
     }
 
 
@@ -381,7 +399,7 @@ public class Response
      * @param request The new associated request
      */
     public void setRequest(org.apache.catalina.connector.Request request) {
-        this.request = (Request) request;
+        this.request = request;
     }
 
 
@@ -413,16 +431,6 @@ public class Response
     }
 
 
-    /**
-     * Set the output stream associated with this Response.
-     *
-     * @param stream The new output stream
-     */
-    public void setStream(OutputStream stream) {
-        // This method is evil
-    }
-
-
     /**
      * Set the suspended flag.
      * 
@@ -506,6 +514,7 @@ public class Response
      * Return the content type that was set or calculated for this response,
      * or <code>null</code> if no content type was set.
      */
+    @Override
     public String getContentType() {
         return (coyoteResponse.getContentType());
     }
@@ -544,6 +553,7 @@ public class Response
      *
      * @exception IOException if an input/output error occurs
      */
+    @Override
     public void flushBuffer() 
         throws IOException {
         outputBuffer.flush();
@@ -553,6 +563,7 @@ public class Response
     /**
      * Return the actual buffer size used for this Response.
      */
+    @Override
     public int getBufferSize() {
         return outputBuffer.getBufferSize();
     }
@@ -561,6 +572,7 @@ public class Response
     /**
      * Return the character encoding used for this Response.
      */
+    @Override
     public String getCharacterEncoding() {
         return (coyoteResponse.getCharacterEncoding());
     }
@@ -573,6 +585,7 @@ public class Response
      *  already been called for this response
      * @exception IOException if an input/output error occurs
      */
+    @Override
     public ServletOutputStream getOutputStream() 
         throws IOException {
 
@@ -592,6 +605,7 @@ public class Response
     /**
      * Return the Locale assigned to this response.
      */
+    @Override
     public Locale getLocale() {
         return (coyoteResponse.getLocale());
     }
@@ -604,6 +618,7 @@ public class Response
      *  already been called for this response
      * @exception IOException if an input/output error occurs
      */
+    @Override
     public PrintWriter getWriter() 
         throws IOException {
 
@@ -611,7 +626,7 @@ public class Response
             throw new IllegalStateException
                 (sm.getString("coyoteResponse.getWriter.ise"));
 
-        if (Globals.STRICT_SERVLET_COMPLIANCE) {
+        if (ENFORCE_ENCODING_IN_GET_WRITER) {
             /*
              * If the response's character encoding has not been specified as
              * described in <code>getCharacterEncoding</code> (i.e., the method
@@ -640,6 +655,7 @@ public class Response
     /**
      * Has the output of this response already been committed?
      */
+    @Override
     public boolean isCommitted() {
         return (coyoteResponse.isCommitted());
     }
@@ -651,6 +667,7 @@ public class Response
      * @exception IllegalStateException if this response has already
      *  been committed
      */
+    @Override
     public void reset() {
 
         if (included)
@@ -670,6 +687,7 @@ public class Response
      * @exception IllegalStateException if the response has already
      *  been committed
      */
+    @Override
     public void resetBuffer() {
         resetBuffer(false);
     }
@@ -711,6 +729,7 @@ public class Response
      * @exception IllegalStateException if this method is called after
      *  output has been committed for this response
      */
+    @Override
     public void setBufferSize(int size) {
 
         if (isCommitted() || !outputBuffer.isNew())
@@ -727,6 +746,7 @@ public class Response
      *
      * @param length The new content length
      */
+    @Override
     public void setContentLength(int length) {
 
         if (isCommitted())
@@ -736,9 +756,6 @@ public class Response
         if (included)
             return;
         
-        if (usingWriter)
-            return;
-        
         coyoteResponse.setContentLength(length);
 
     }
@@ -749,6 +766,8 @@ public class Response
      *
      * @param type The new content type
      */
+    @Override
+    @SuppressWarnings("deprecation") // isSpace (deprecated) cannot be replaced by isWhiteSpace
     public void setContentType(String type) {
 
         if (isCommitted())
@@ -776,6 +795,7 @@ public class Response
             if (index != -1) {
                 int len = type.length();
                 index++;
+                // N.B. isSpace (deprecated) cannot be replaced by isWhiteSpace
                 while (index < len && Character.isSpace(type.charAt(index))) {
                     index++;
                 }
@@ -800,8 +820,9 @@ public class Response
      * of the request. This method must be called prior to reading
      * request parameters or reading input using getReader().
      *
-     * @param charset String containing the name of the chararacter encoding.
+     * @param charset String containing the name of the character encoding.
      */
+    @Override
     public void setCharacterEncoding(String charset) {
 
         if (isCommitted())
@@ -820,14 +841,14 @@ public class Response
         isCharacterEncodingSet = true;
     }
 
-    
-    
+
     /**
      * Set the Locale that is appropriate for this response, including
      * setting the appropriate character encoding.
      *
      * @param locale The new locale
      */
+    @Override
     public void setLocale(Locale locale) {
 
         if (isCommitted())
@@ -860,39 +881,31 @@ public class Response
     // --------------------------------------------------- HttpResponse Methods
 
 
-    /**
-     * Return an array of all cookies set for this response, or
-     * a zero-length array if no cookies have been set.
-     */
-    public Cookie[] getCookies() {
-        return cookies.toArray(new Cookie[cookies.size()]);
-    }
-
-
     /**
      * Return the value for the specified header, or <code>null</code> if this
      * header has not been set.  If more than one value was added for this
-     * name, only the first is returned; use getHeaderValues() to retrieve all
-     * of them.
+     * name, only the first is returned; use {@link #getHeaders(String)} to
+     * retrieve all of them.
      *
      * @param name Header name to look up
      */
+    @Override
     public String getHeader(String name) {
         return coyoteResponse.getMimeHeaders().getHeader(name);
     }
 
 
     /**
-     * Return an array of all the header names set for this response, or
-     * a zero-length array if no headers have been set.
+     * Return an Iterable of all the header names set for this response.
      */
-    public String[] getHeaderNames() {
+    @Override
+    public Collection<String> getHeaderNames() {
 
         MimeHeaders headers = coyoteResponse.getMimeHeaders();
         int n = headers.size();
-        String[] result = new String[n];
+        List<String> result = new ArrayList<String>(n);
         for (int i = 0; i < n; i++) {
-            result[i] = headers.getName(i).toString();
+            result.add(headers.getName(i).toString());
         }
         return result;
 
@@ -900,23 +913,21 @@ public class Response
 
 
     /**
-     * Return an array of all the header values associated with the
-     * specified header name, or an zero-length array if there are no such
-     * header values.
+     * Return a Collection of all the header values associated with the
+     * specified header name.
      *
      * @param name Header name to look up
      */
-    public String[] getHeaderValues(String name) {
+    @Override
+    public Collection<String> getHeaders(String name) {
 
-        Enumeration enumeration = coyoteResponse.getMimeHeaders().values(name);
-        Vector result = new Vector();
+        Enumeration<String> enumeration =
+            coyoteResponse.getMimeHeaders().values(name);
+        Vector<String> result = new Vector<String>();
         while (enumeration.hasMoreElements()) {
             result.addElement(enumeration.nextElement());
         }
-        String[] resultArray = new String[result.size()];
-        result.copyInto(resultArray);
-        return resultArray;
-
+        return result;
     }
 
 
@@ -932,6 +943,7 @@ public class Response
     /**
      * Return the HTTP status code associated with this Response.
      */
+    @Override
     public int getStatus() {
         return coyoteResponse.getStatus();
     }
@@ -959,14 +971,19 @@ public class Response
      *
      * @param cookie Cookie to be added
      */
+    @Override
     public void addCookie(final Cookie cookie) {
 
         // Ignore any call from an included servlet
-        if (included)
+        if (included || isCommitted())
             return;
 
-        addCookieInternal(cookie);
-
+        final StringBuffer sb = generateCookieString(cookie);
+        //if we reached here, no exception, cookie is valid
+        // the header name is Set-Cookie for both "old" and v.1 ( RFC2109 )
+        // RFC2965 is not supported by browsers and the Servlet spec
+        // asks for 2109.
+        addHeader("Set-Cookie", sb.toString());
     }
 
     /**
@@ -974,88 +991,46 @@ public class Response
      * any previous 
      * @param cookie
      */
-    public void addSessionCookieInternal(final Cookie cookie,
-            boolean httpOnly) {
+    public void addSessionCookieInternal(final Cookie cookie) {
         if (isCommitted())
             return;
         
         String name = cookie.getName();
         final String headername = "Set-Cookie";
         final String startsWith = name + "=";
-        final StringBuffer sb = generateCookieString(cookie, httpOnly);
+        final StringBuffer sb = generateCookieString(cookie);
         boolean set = false;
         MimeHeaders headers = coyoteResponse.getMimeHeaders();
         int n = headers.size();
         for (int i = 0; i < n; i++) {
             if (headers.getName(i).toString().equals(headername)) {
                 if (headers.getValue(i).toString().startsWith(startsWith)) {
-                    headers.setValue(sb.toString());
+                    headers.getValue(i).setString(sb.toString());
                     set = true;
                 }
             }
         }
-        if (set) {
-            Iterator<Cookie> iter = cookies.iterator();
-            while (iter.hasNext()) {
-                Cookie c = iter.next();
-                if (name.equals(c.getName())) {
-                    iter.remove();
-                    break;
-                }
-            }
-        } else {
+        if (!set) {
             addHeader(headername, sb.toString());
         }
-        cookies.add(cookie);
         
         
     }
-    /**
-     * Add the specified Cookie to those that will be included with
-     * this Response.
-     *
-     * @param cookie Cookie to be added
-     */
-    public void addCookieInternal(final Cookie cookie) {
-        addCookieInternal(cookie, false);
-    }
 
-    /**
-     * Add the specified Cookie to those that will be included with
-     * this Response.
-     *
-     * @param cookie    Cookie to be added
-     * @param httpOnly  Should the httpOnly falg be set on this cookie
-     */
-    public void addCookieInternal(final Cookie cookie, final boolean httpOnly) {
-
-        if (isCommitted())
-            return;
-
-        final StringBuffer sb = generateCookieString(cookie, httpOnly);
-        //if we reached here, no exception, cookie is valid
-        // the header name is Set-Cookie for both "old" and v.1 ( RFC2109 )
-        // RFC2965 is not supported by browsers and the Servlet spec
-        // asks for 2109.
-        addHeader("Set-Cookie", sb.toString());
-
-        cookies.add(cookie);
-    }
-
-    public StringBuffer generateCookieString(final Cookie cookie, 
-            final boolean httpOnly) {
+    public StringBuffer generateCookieString(final Cookie cookie) {
         final StringBuffer sb = new StringBuffer();
         //web application code can receive a IllegalArgumentException 
-        //from the appendCookieValue invokation
+        //from the appendCookieValue invocation
         if (SecurityUtil.isPackageProtectionEnabled()) {
-            AccessController.doPrivileged(new PrivilegedAction() {
-                public Object run(){
+            AccessController.doPrivileged(new PrivilegedAction<Void>() {
+                @Override
+                public Void run(){
                     ServerCookie.appendCookieValue
                         (sb, cookie.getVersion(), cookie.getName(), 
                          cookie.getValue(), cookie.getPath(), 
                          cookie.getDomain(), cookie.getComment(), 
                          cookie.getMaxAge(), cookie.getSecure(),
-                         httpOnly);
+                         cookie.isHttpOnly());
                     return null;
                 }
             });
@@ -1063,7 +1038,8 @@ public class Response
             ServerCookie.appendCookieValue
                 (sb, cookie.getVersion(), cookie.getName(), cookie.getValue(),
                      cookie.getPath(), cookie.getDomain(), cookie.getComment(), 
-                     cookie.getMaxAge(), cookie.getSecure(), httpOnly);
+                     cookie.getMaxAge(), cookie.getSecure(),
+                     cookie.isHttpOnly());
         }
         return sb;
     }
@@ -1075,6 +1051,7 @@ public class Response
      * @param name Name of the header to set
      * @param value Date value to be set
      */
+    @Override
     public void addDateHeader(String name, long value) {
 
         if (name == null || name.length() == 0) {
@@ -1106,6 +1083,7 @@ public class Response
      * @param name Name of the header to set
      * @param value Value to be set
      */
+    @Override
     public void addHeader(String name, String value) {
 
         if (name == null || name.length() == 0 || value == null) {
@@ -1130,6 +1108,7 @@ public class Response
      * @param name Name of the header to set
      * @param value Integer value to be set
      */
+    @Override
     public void addIntHeader(String name, int value) {
 
         if (name == null || name.length() == 0) {
@@ -1153,6 +1132,7 @@ public class Response
      *
      * @param name Name of the header to check
      */
+    @Override
     public boolean containsHeader(String name) {
         // Need special handling for Content-Type and Content-Length due to
         // special handling of these in coyoteResponse
@@ -1178,6 +1158,7 @@ public class Response
      *
      * @param url URL to be encoded
      */
+    @Override
     public String encodeRedirectURL(String url) {
 
         if (isEncodeable(toAbsolute(url))) {
@@ -1198,6 +1179,8 @@ public class Response
      * @deprecated As of Version 2.1 of the Java Servlet API, use
      *  <code>encodeRedirectURL()</code> instead.
      */
+    @Override
+    @Deprecated
     public String encodeRedirectUrl(String url) {
         return (encodeRedirectURL(url));
     }
@@ -1209,6 +1192,7 @@ public class Response
      *
      * @param url URL to be encoded
      */
+    @Override
     public String encodeURL(String url) {
         
         String absolute = toAbsolute(url);
@@ -1234,6 +1218,8 @@ public class Response
      * @deprecated As of Version 2.1 of the Java Servlet API, use
      *  <code>encodeURL()</code> instead.
      */
+    @Override
+    @Deprecated
     public String encodeUrl(String url) {
         return (encodeURL(url));
     }
@@ -1269,6 +1255,7 @@ public class Response
      *  already been committed
      * @exception IOException if an input/output error occurs
      */
+    @Override
     public void sendError(int status) 
         throws IOException {
         sendError(status, null);
@@ -1285,6 +1272,7 @@ public class Response
      *  already been committed
      * @exception IOException if an input/output error occurs
      */
+    @Override
     public void sendError(int status, String message) 
         throws IOException {
 
@@ -1324,6 +1312,7 @@ public class Response
      *  already been committed
      * @exception IOException if an input/output error occurs
      */
+    @Override
     public void sendRedirect(String location) 
         throws IOException {
 
@@ -1359,6 +1348,7 @@ public class Response
      * @param name Name of the header to set
      * @param value Date value to be set
      */
+    @Override
     public void setDateHeader(String name, long value) {
 
         if (name == null || name.length() == 0) {
@@ -1390,6 +1380,7 @@ public class Response
      * @param name Name of the header to set
      * @param value Value to be set
      */
+    @Override
     public void setHeader(String name, String value) {
 
         if (name == null || name.length() == 0 || value == null) {
@@ -1414,6 +1405,7 @@ public class Response
      * @param name Name of the header to set
      * @param value Integer value to be set
      */
+    @Override
     public void setIntHeader(String name, int value) {
 
         if (name == null || name.length() == 0) {
@@ -1437,6 +1429,7 @@ public class Response
      *
      * @param status The new HTTP status
      */
+    @Override
     public void setStatus(int status) {
         setStatus(status, null);
     }
@@ -1452,6 +1445,8 @@ public class Response
      *  has been deprecated due to the ambiguous meaning of the message
      *  parameter.
      */
+    @Override
+    @Deprecated
     public void setStatus(int status, String message) {
 
         if (isCommitted())
@@ -1500,12 +1495,18 @@ public class Response
         if (hreq.isRequestedSessionIdFromCookie())
             return (false);
         
-        if (SecurityUtil.isPackageProtectionEnabled()) {
-            return ((Boolean)
-                AccessController.doPrivileged(new PrivilegedAction() {
+        // Is URL encoding permitted
+        if (!hreq.getServletContext().getEffectiveSessionTrackingModes().
+                contains(SessionTrackingMode.URL))
+            return false;
 
-                public Object run(){
-                    return new Boolean(doIsEncodeable(hreq, session, location));
+        if (SecurityUtil.isPackageProtectionEnabled()) {
+            return (
+                AccessController.doPrivileged(new PrivilegedAction<Boolean>() {
+
+                @Override
+                public Boolean run(){
+                    return Boolean.valueOf(doIsEncodeable(hreq, session, location));
                 }
             })).booleanValue();
         } else {
@@ -1550,7 +1551,10 @@ public class Response
             String file = url.getFile();
             if ((file == null) || !file.startsWith(contextPath))
                 return (false);
-            String tok = ";" + Globals.SESSION_PARAMETER_NAME + "=" + session.getIdInternal();
+            String tok = ";" +
+                    ApplicationSessionCookieConfig.getSessionUriParamName(
+                                request.getContext()) +
+                    "=" + session.getIdInternal();
             if( file.indexOf(tok, contextPath.length()) >= 0 )
                 return (false);
         }
@@ -1605,9 +1609,10 @@ public class Response
                     final String frelativePath = relativePath;
                     if (SecurityUtil.isPackageProtectionEnabled() ){
                         try{
-                            encodedURI = (String)AccessController.doPrivileged( 
-                                new PrivilegedExceptionAction(){                                
-                                    public Object run() throws IOException{
+                            encodedURI = AccessController.doPrivileged( 
+                                new PrivilegedExceptionAction<String>(){                                
+                                    @Override
+                                    public String run() throws IOException{
                                         return urlEncoder.encodeURL(frelativePath);
                                     }
                            });   
@@ -1683,10 +1688,11 @@ public class Response
             anchor = path.substring(pound);
             path = path.substring(0, pound);
         }
-        StringBuffer sb = new StringBuffer(path);
+        StringBuilder sb = new StringBuilder(path);
         if( sb.length() > 0 ) { // jsessionid can't be first.
             sb.append(";");
-            sb.append(Globals.SESSION_PARAMETER_NAME);
+            sb.append(ApplicationSessionCookieConfig.getSessionUriParamName(
+                    request.getContext()));
             sb.append("=");
             sb.append(sessionId);
         }
