@@ -21,34 +21,31 @@ import java.io.Externalizable;
 import java.io.IOException;
 import java.io.NotSerializableException;
 import java.io.ObjectInput;
-import java.io.ObjectOutput;
 import java.io.ObjectInputStream;
+import java.io.ObjectOutput;
 import java.io.ObjectOutputStream;
 import java.io.Serializable;
 import java.security.Principal;
 import java.util.ArrayList;
-import java.util.Enumeration;
-import java.util.HashMap;
 import java.util.Hashtable;
+import java.util.concurrent.ConcurrentHashMap;
+import java.util.concurrent.atomic.AtomicInteger;
 import java.util.concurrent.locks.Lock;
 import java.util.concurrent.locks.ReentrantReadWriteLock;
-import javax.servlet.http.HttpSession;
-import javax.servlet.http.HttpSessionContext;
 
 import org.apache.catalina.Manager;
-import org.apache.catalina.ha.ClusterManager;
+import org.apache.catalina.SessionListener;
 import org.apache.catalina.ha.CatalinaCluster;
-import org.apache.catalina.ha.ClusterSession;
+import org.apache.catalina.ha.ClusterManager;
 import org.apache.catalina.ha.ClusterMessage;
+import org.apache.catalina.ha.ClusterSession;
 import org.apache.catalina.realm.GenericPrincipal;
+import org.apache.catalina.session.ManagerBase;
+import org.apache.catalina.session.StandardManager;
 import org.apache.catalina.session.StandardSession;
 import org.apache.catalina.tribes.io.ReplicationStream;
 import org.apache.catalina.tribes.tipis.ReplicatedMapEntry;
-import org.apache.catalina.util.Enumerator;
-import org.apache.catalina.util.StringManager;
-import org.apache.catalina.session.StandardManager;
-import org.apache.catalina.session.ManagerBase;
-import java.util.concurrent.atomic.AtomicInteger;
+import org.apache.tomcat.util.res.StringManager;
 
 /**
  *
@@ -56,17 +53,17 @@ import java.util.concurrent.atomic.AtomicInteger;
  * track of deltas during a request.
  *
  * @author Filip Hanik
- * @version $Id: DeltaSession.java 1195604 2011-10-31 18:42:09Z kkolinko $
+ * @version $Id: DeltaSession.java 1099772 2011-05-05 12:38:44Z markt $
  */
 
 public class DeltaSession extends StandardSession implements Externalizable,ClusterSession,ReplicatedMapEntry {
 
-    public static org.apache.juli.logging.Log log = org.apache.juli.logging.LogFactory.getLog(DeltaSession.class);
+    public static final org.apache.juli.logging.Log log = org.apache.juli.logging.LogFactory.getLog(DeltaSession.class);
 
     /**
      * The string manager for this package.
      */
-    protected static StringManager sm = StringManager.getManager(Constants.Package);
+    protected static final StringManager sm = StringManager.getManager(Constants.Package);
 
     // ----------------------------------------------------- Instance Variables
 
@@ -85,7 +82,7 @@ public class DeltaSession extends StandardSession implements Externalizable,Clus
     private transient DeltaRequest deltaRequest = null;
 
     /**
-     * Last time the session was replicatd, used for distributed expiring of
+     * Last time the session was replicated, used for distributed expiring of
      * session
      */
     private transient long lastTimeReplicated = System.currentTimeMillis();
@@ -97,13 +94,16 @@ public class DeltaSession extends StandardSession implements Externalizable,Clus
 
     // ----------------------------------------------------------- Constructors
 
-    /**
-     * Construct a new Session.
-     */
     public DeltaSession() {
         this(null);
     }
 
+    /**
+     * Construct a new Session associated with the specified Manager.
+     *
+     * @param manager
+     *            The manager with which this Session is associated
+     */
     public DeltaSession(Manager manager) {
         super(manager);
         this.resetDeltaRequest();
@@ -116,6 +116,7 @@ public class DeltaSession extends StandardSession implements Externalizable,Clus
          * and is not in a locked state
          * @return boolean
          */
+        @Override
         public boolean isDirty() {
             return getDeltaRequest().getSize()>0;
         }
@@ -125,6 +126,7 @@ public class DeltaSession extends StandardSession implements Externalizable,Clus
          * Otherwise it will serialize the entire object.
          * @return boolean
          */
+        @Override
         public boolean isDiffable() {
             return true;
         }
@@ -134,6 +136,7 @@ public class DeltaSession extends StandardSession implements Externalizable,Clus
          * @return byte[]
          * @throws IOException
          */
+        @Override
         public byte[] getDiff() throws IOException {
             try{
                 lock();
@@ -163,6 +166,7 @@ public class DeltaSession extends StandardSession implements Externalizable,Clus
          * @param length int
          * @throws IOException
          */
+        @Override
         public void applyDiff(byte[] diff, int offset, int length) throws IOException, ClassNotFoundException {
             try {
                 lock();
@@ -185,6 +189,7 @@ public class DeltaSession extends StandardSession implements Externalizable,Clus
         /**
          * Resets the current diff state and resets the dirty flag
          */
+        @Override
         public void resetDiff() {
             resetDeltaRequest();
         }
@@ -192,6 +197,7 @@ public class DeltaSession extends StandardSession implements Externalizable,Clus
         /**
          * Lock during serialization
          */
+        @Override
         public void lock() {
             diffLock.lock();
         }
@@ -199,10 +205,12 @@ public class DeltaSession extends StandardSession implements Externalizable,Clus
         /**
          * Unlock after serialization
          */
+        @Override
         public void unlock() {
             diffLock.unlock();
         }
 
+        @Override
         public void setOwner(Object owner) {
             if ( owner instanceof ClusterManager && getManager()==null) {
                 ClusterManager cm = (ClusterManager)owner;
@@ -220,6 +228,7 @@ public class DeltaSession extends StandardSession implements Externalizable,Clus
      * returns true if this session is the primary session, if that is the case,
      * the manager can expire it upon timeout.
      */
+    @Override
     public boolean isPrimarySession() {
         return isPrimarySession;
     }
@@ -230,6 +239,7 @@ public class DeltaSession extends StandardSession implements Externalizable,Clus
      * @param primarySession
      *            Flag value
      */
+    @Override
     public void setPrimarySession(boolean primarySession) {
         this.isPrimarySession = primarySession;
     }
@@ -251,26 +261,19 @@ public class DeltaSession extends StandardSession implements Externalizable,Clus
      * @param id
      *            The new session identifier
      */
+    @Override
     public void setId(String id) {
-        super.setId(id, true);
+        super.setId(id);
         resetDeltaRequest();
     }
 
-   
 
-    /**
-     * Return the last client access time without invalidation check
-     * @see #getLastAccessedTime().
-     */
-    public long getLastAccessedTimeInternal() {
-        return (this.lastAccessedTime);
-    }
-
-
-   
+    @Override
     public void setMaxInactiveInterval(int interval) {
         this.setMaxInactiveInterval(interval,true);
     }
+
+
     public void setMaxInactiveInterval(int interval, boolean addDeltaRequest) {
         super.maxInactiveInterval = interval;
         if (isValid && interval == 0) {
@@ -284,7 +287,6 @@ public class DeltaSession extends StandardSession implements Externalizable,Clus
                     unlock();
                 }
             }
-                
         }
     }
 
@@ -294,6 +296,7 @@ public class DeltaSession extends StandardSession implements Externalizable,Clus
      * @param isNew
      *            The new value for the <code>isNew</code> flag
      */
+    @Override
     public void setNew(boolean isNew) {
         setNew(isNew, true);
     }
@@ -319,6 +322,7 @@ public class DeltaSession extends StandardSession implements Externalizable,Clus
      * @param principal
      *            The new Principal, or <code>null</code> if none
      */
+    @Override
     public void setPrincipal(Principal principal) {
         setPrincipal(principal, true);
     }
@@ -359,6 +363,7 @@ public class DeltaSession extends StandardSession implements Externalizable,Clus
     /**
      * Return the <code>isValid</code> flag for this session.
      */
+    @Override
     public boolean isValid() {
         if (this.expiring) {
             return true;
@@ -371,7 +376,12 @@ public class DeltaSession extends StandardSession implements Externalizable,Clus
         }
         if (maxInactiveInterval >= 0) {
             long timeNow = System.currentTimeMillis();
-            int timeIdle = (int) ( (timeNow - thisAccessedTime) / 1000L);
+            int timeIdle;
+            if (LAST_ACCESS_AT_START) {
+                timeIdle = (int) ((timeNow - lastAccessedTime) / 1000L);
+            } else {
+                timeIdle = (int) ((timeNow - thisAccessedTime) / 1000L);
+            }
             if (isPrimarySession()) {
                 if (timeIdle >= maxInactiveInterval) {
                     expire(true);
@@ -392,14 +402,14 @@ public class DeltaSession extends StandardSession implements Externalizable,Clus
     /**
      * End the access and register to ReplicationValve (crossContext support)
      */
+    @Override
     public void endAccess() {
-    	super.endAccess() ;
-    	if(manager instanceof DeltaManager) {
-            ((DeltaManager)manager).registerSessionAtReplicationValve(this);   	
-    	}
+        super.endAccess() ;
+        if(manager instanceof DeltaManager) {
+            ((DeltaManager)manager).registerSessionAtReplicationValve(this);
+        }
     }
     
-    
     // ------------------------------------------------- Session Public Methods
 
     /**
@@ -409,6 +419,7 @@ public class DeltaSession extends StandardSession implements Externalizable,Clus
      * @param notify
      *            Should we notify listeners about the demise of this session?
      */
+    @Override
     public void expire(boolean notify) {
         expire(notify, true);
     }
@@ -418,17 +429,13 @@ public class DeltaSession extends StandardSession implements Externalizable,Clus
             return;
         String expiredId = getIdInternal();
 
-        if(notifyCluster && expiredId != null && manager != null &&
+        if(expiredId != null && manager != null &&
            manager instanceof DeltaManager) {
             DeltaManager dmanager = (DeltaManager)manager;
             CatalinaCluster cluster = dmanager.getCluster();
             ClusterMessage msg = dmanager.requestCompleted(expiredId, true);
             if (msg != null) {
-                if(dmanager.doDomainReplication()) {
-                    cluster.sendClusterDomain(msg);
-                } else {
-                    cluster.send(msg);
-                }
+                cluster.send(msg);
             }
         }
 
@@ -438,7 +445,7 @@ public class DeltaSession extends StandardSession implements Externalizable,Clus
             if (log.isDebugEnabled())
                 log.debug(sm.getString("deltaSession.notifying",
                                        ((ClusterManager)manager).getName(), 
-                                       new Boolean(isPrimarySession()), 
+                                       Boolean.valueOf(isPrimarySession()), 
                                        expiredId));
             if ( manager instanceof DeltaManager ) {
                 ( (DeltaManager) manager).sessionExpired(expiredId);
@@ -450,6 +457,7 @@ public class DeltaSession extends StandardSession implements Externalizable,Clus
      * Release all object references, and initialize instance variables, in
      * preparation for reuse of this object.
      */
+    @Override
     public void recycle() {
         try {
             lock();
@@ -464,8 +472,9 @@ public class DeltaSession extends StandardSession implements Externalizable,Clus
     /**
      * Return a string representation of this object.
      */
+    @Override
     public String toString() {
-        StringBuffer sb = new StringBuffer();
+        StringBuilder sb = new StringBuilder();
         sb.append("DeltaSession[");
         sb.append(id);
         sb.append("]");
@@ -474,6 +483,7 @@ public class DeltaSession extends StandardSession implements Externalizable,Clus
 
     // ------------------------------------------------ Session Package Methods
 
+    @Override
     public void readExternal(ObjectInput in) throws IOException,ClassNotFoundException {
         try {
             lock();
@@ -548,38 +558,6 @@ public class DeltaSession extends StandardSession implements Externalizable,Clus
     // ----------------------------------------------HttpSession Public Methods
 
 
-
-    /**
-     * Check whether the Object can be distributed.
-     * The object is always distributable, if the cluster manager
-     * decides to never distribute it.
-     * @param name The name of the attribute to check
-     * @param value The value of the attribute to check
-     * @return true if the attribute is distributable, false otherwise
-     */
-    @Override
-    protected boolean isAttributeDistributable(String name, Object value) {
-        if (manager instanceof ClusterManagerBase &&
-            !((ClusterManagerBase)manager).willAttributeDistribute(name))
-            return true;
-        return super.isAttributeDistributable(name, value);
-    }
-
-    /**
-     * Exclude attributes from replication.
-     * @param name the attribute's name
-     * @return true if attribute should not be replicated
-     */
-    @Override
-    protected boolean exclude(String name) {
-
-        if (super.exclude(name))
-            return true;
-        if (manager instanceof ClusterManagerBase)
-            return !((ClusterManagerBase)manager).willAttributeDistribute(name);
-        return false;
-    }
-
     /**
      * Remove the object bound with the specified name from this session. If the
      * session does not have an object bound with this name, this method does
@@ -598,6 +576,7 @@ public class DeltaSession extends StandardSession implements Externalizable,Clus
      * @exception IllegalStateException
      *                if this method is called on an invalidated session
      */
+    @Override
     public void removeAttribute(String name, boolean notify) {
         removeAttribute(name, notify, true);
     }
@@ -627,6 +606,7 @@ public class DeltaSession extends StandardSession implements Externalizable,Clus
      * @exception IllegalStateException
      *                if this method is called on an invalidated session
      */
+    @Override
     public void setAttribute(String name, Object value) {
         setAttribute(name, value, true, true);
     }
@@ -655,7 +635,6 @@ public class DeltaSession extends StandardSession implements Externalizable,Clus
 
     // -------------------------------------------- HttpSession Private Methods
 
-
     /**
      * Read a serialized version of this session object from the specified
      * object input stream.
@@ -690,7 +669,7 @@ public class DeltaSession extends StandardSession implements Externalizable,Clus
         boolean hasPrincipal = stream.readBoolean();
         principal = null;
         if (hasPrincipal) {
-            principal = SerializablePrincipal.readPrincipal(stream,getManager().getContainer().getRealm());
+            principal = SerializablePrincipal.readPrincipal(stream);
         }
 
         //        setId((String) stream.readObject());
@@ -698,13 +677,13 @@ public class DeltaSession extends StandardSession implements Externalizable,Clus
         if (log.isDebugEnabled()) log.debug(sm.getString("deltaSession.readSession", id));
 
         // Deserialize the attribute count and attribute values
-        if (attributes == null) attributes = new Hashtable();
+        if (attributes == null) attributes = new ConcurrentHashMap<String, Object>();
         int n = ( (Integer) stream.readObject()).intValue();
         boolean isValidSave = isValid;
         isValid = true;
         for (int i = 0; i < n; i++) {
             String name = (String) stream.readObject();
-            Object value = (Object) stream.readObject();
+            Object value = stream.readObject();
             if ( (value instanceof String) && (value.equals(NOT_SERIALIZED)))
                 continue;
             attributes.put(name, value);
@@ -712,15 +691,18 @@ public class DeltaSession extends StandardSession implements Externalizable,Clus
         isValid = isValidSave;
 
         if (listeners == null) {
-            listeners = new ArrayList();
+            ArrayList<SessionListener> arrayList =
+                new ArrayList<SessionListener>();
+            listeners = arrayList;
         }
 
         if (notes == null) {
-            notes = new Hashtable();
+            notes = new Hashtable<String,Object>();
         }
         activate();
     }
 
+    @Override
     public void writeExternal(ObjectOutput out ) throws java.io.IOException {
         try {
             lock();
@@ -752,19 +734,20 @@ public class DeltaSession extends StandardSession implements Externalizable,Clus
      * @exception IOException
      *                if an input/output error occurs
      */
+    @Override
     protected void writeObject(ObjectOutputStream stream) throws IOException {
         writeObject((ObjectOutput)stream);
     }
     
     private void writeObject(ObjectOutput stream) throws IOException {
         // Write the scalar instance variables (except Manager)
-        stream.writeObject(new Long(creationTime));
-        stream.writeObject(new Long(lastAccessedTime));
-        stream.writeObject(new Integer(maxInactiveInterval));
-        stream.writeObject(new Boolean(isNew));
-        stream.writeObject(new Boolean(isValid));
-        stream.writeObject(new Long(thisAccessedTime));
-        stream.writeObject(new Long(version));
+        stream.writeObject(Long.valueOf(creationTime));
+        stream.writeObject(Long.valueOf(lastAccessedTime));
+        stream.writeObject(Integer.valueOf(maxInactiveInterval));
+        stream.writeObject(Boolean.valueOf(isNew));
+        stream.writeObject(Boolean.valueOf(isValid));
+        stream.writeObject(Long.valueOf(thisAccessedTime));
+        stream.writeObject(Long.valueOf(version));
         stream.writeBoolean(getPrincipal() != null);
         if (getPrincipal() != null) {
             SerializablePrincipal.writePrincipal((GenericPrincipal) principal,stream);
@@ -775,8 +758,8 @@ public class DeltaSession extends StandardSession implements Externalizable,Clus
 
         // Accumulate the names of serializable and non-serializable attributes
         String keys[] = keys();
-        ArrayList saveNames = new ArrayList();
-        ArrayList saveValues = new ArrayList();
+        ArrayList<String> saveNames = new ArrayList<String>();
+        ArrayList<Object> saveValues = new ArrayList<Object>();
         for (int i = 0; i < keys.length; i++) {
             Object value = null;
             value = attributes.get(keys[i]);
@@ -790,9 +773,9 @@ public class DeltaSession extends StandardSession implements Externalizable,Clus
 
         // Serialize the attribute count and the Serializable attributes
         int n = saveNames.size();
-        stream.writeObject(new Integer(n));
+        stream.writeObject(Integer.valueOf(n));
         for (int i = 0; i < n; i++) {
-            stream.writeObject( (String) saveNames.get(i));
+            stream.writeObject( saveNames.get(i));
             try {
                 stream.writeObject(saveValues.get(i));
             } catch (NotSerializableException e) {
@@ -806,7 +789,6 @@ public class DeltaSession extends StandardSession implements Externalizable,Clus
 
     // -------------------------------------------------------- Private Methods
 
-    
 
     /**
      * Return the value of an attribute without a check for validity.
@@ -837,6 +819,7 @@ public class DeltaSession extends StandardSession implements Externalizable,Clus
         return lastTimeReplicated;
     }
 
+    @Override
     public long getVersion() {
         return version;
     }
@@ -845,6 +828,7 @@ public class DeltaSession extends StandardSession implements Externalizable,Clus
         this.lastTimeReplicated = lastTimeReplicated;
     }
 
+    @Override
     public void setVersion(long version) {
         this.version = version;
     }
@@ -854,50 +838,3 @@ public class DeltaSession extends StandardSession implements Externalizable,Clus
         if ( accessCount != null ) super.accessCount.set(count);
     }
 }
-
-// -------------------------------------------------------------- Private Class
-
-/**
- * This class is a dummy implementation of the <code>HttpSessionContext</code>
- * interface, to conform to the requirement that such an object be returned when
- * <code>HttpSession.getSessionContext()</code> is called.
- *
- * @author Craig R. McClanahan
- *
- * @deprecated As of Java Servlet API 2.1 with no replacement. The interface
- *             will be removed in a future version of this API.
- */
-
-final class StandardSessionContext
-    implements HttpSessionContext {
-
-    private HashMap dummy = new HashMap();
-
-    /**
-     * Return the session identifiers of all sessions defined within this
-     * context.
-     *
-     * @deprecated As of Java Servlet API 2.1 with no replacement. This method
-     *             must return an empty <code>Enumeration</code> and will be
-     *             removed in a future version of the API.
-     */
-    public Enumeration getIds() {
-        return (new Enumerator(dummy));
-    }
-
-    /**
-     * Return the <code>HttpSession</code> associated with the specified
-     * session identifier.
-     *
-     * @param id
-     *            Session identifier for which to look up a session
-     *
-     * @deprecated As of Java Servlet API 2.1 with no replacement. This method
-     *             must return null and will be removed in a future version of
-     *             the API.
-     */
-    public HttpSession getSession(String id) {
-        return (null);
-    }
-
-}
