@@ -19,12 +19,14 @@ package org.apache.catalina.mbeans;
 
 
 import java.util.Iterator;
+
 import javax.naming.Binding;
 import javax.naming.Context;
 import javax.naming.InitialContext;
 import javax.naming.NamingEnumeration;
 import javax.naming.NamingException;
 import javax.naming.OperationNotSupportedException;
+
 import org.apache.catalina.Group;
 import org.apache.catalina.Lifecycle;
 import org.apache.catalina.LifecycleEvent;
@@ -43,13 +45,13 @@ import org.apache.tomcat.util.modeler.Registry;
  * management.
  *
  * @author Craig R. McClanahan
- * @version $Id: GlobalResourcesLifecycleListener.java 939518 2010-04-30 00:08:58Z kkolinko $
+ * @version $Id: GlobalResourcesLifecycleListener.java 987920 2010-08-22 15:34:34Z markt $
  * @since 4.1
  */
 
 public class GlobalResourcesLifecycleListener
     implements LifecycleListener {
-    private static Log log = LogFactory.getLog(GlobalResourcesLifecycleListener.class);
+    private static final Log log = LogFactory.getLog(GlobalResourcesLifecycleListener.class);
 
     // ----------------------------------------------------- Instance Variables
 
@@ -132,9 +134,9 @@ public class GlobalResourcesLifecycleListener
         }
 
         try {
-            NamingEnumeration bindings = context.listBindings("");
+            NamingEnumeration<Binding> bindings = context.listBindings("");
             while (bindings.hasMore()) {
-                Binding binding = (Binding) bindings.next();
+                Binding binding = bindings.next();
                 String name = prefix + binding.getName();
                 Object value = context.lookup(binding.getName());
                 if (log.isDebugEnabled()) {
@@ -182,9 +184,9 @@ public class GlobalResourcesLifecycleListener
         }
 
         // Create the MBeans for each defined Role
-        Iterator roles = database.getRoles();
+        Iterator<Role> roles = database.getRoles();
         while (roles.hasNext()) {
-            Role role = (Role) roles.next();
+            Role role = roles.next();
             if (log.isDebugEnabled()) {
                 log.debug("  Creating Role MBean for role " + role);
             }
@@ -195,9 +197,9 @@ public class GlobalResourcesLifecycleListener
         }
 
         // Create the MBeans for each defined Group
-        Iterator groups = database.getGroups();
+        Iterator<Group> groups = database.getGroups();
         while (groups.hasNext()) {
-            Group group = (Group) groups.next();
+            Group group = groups.next();
             if (log.isDebugEnabled()) {
                 log.debug("  Creating Group MBean for group " + group);
             }
@@ -208,9 +210,9 @@ public class GlobalResourcesLifecycleListener
         }
 
         // Create the MBeans for each defined User
-        Iterator users = database.getUsers();
+        Iterator<User> users = database.getUsers();
         while (users.hasNext()) {
-            User user = (User) users.next();
+            User user = users.next();
             if (log.isDebugEnabled()) {
                 log.debug("  Creating User MBean for user " + user);
             }
