@@ -27,15 +27,18 @@ import java.io.OutputStreamWriter;
 import java.io.PrintWriter;
 import java.util.HashMap;
 import java.util.Iterator;
+
+import org.apache.catalina.Globals;
 import org.apache.catalina.Group;
 import org.apache.catalina.Role;
 import org.apache.catalina.User;
 import org.apache.catalina.UserDatabase;
-import org.apache.catalina.util.StringManager;
 import org.apache.juli.logging.Log;
 import org.apache.juli.logging.LogFactory;
+import org.apache.tomcat.util.ExceptionUtils;
+import org.apache.tomcat.util.digester.AbstractObjectCreationFactory;
 import org.apache.tomcat.util.digester.Digester;
-import org.apache.tomcat.util.digester.ObjectCreationFactory;
+import org.apache.tomcat.util.res.StringManager;
 import org.xml.sax.Attributes;
 
 
@@ -45,14 +48,14 @@ import org.xml.sax.Attributes;
  * and uses a specified XML file for its persistent storage.</p>
  *
  * @author Craig R. McClanahan
- * @version $Id: MemoryUserDatabase.java 1140071 2011-06-27 09:27:20Z markt $
+ * @version $Id: MemoryUserDatabase.java 1095677 2011-04-21 12:25:31Z markt $
  * @since 4.1
  */
 
 public class MemoryUserDatabase implements UserDatabase {
 
 
-    private static Log log = LogFactory.getLog(MemoryUserDatabase.class);
+    private static final Log log = LogFactory.getLog(MemoryUserDatabase.class);
 
     // ----------------------------------------------------------- Constructors
 
@@ -87,7 +90,7 @@ public class MemoryUserDatabase implements UserDatabase {
      * The set of {@link Group}s defined in this database, keyed by
      * group name.
      */
-    protected HashMap groups = new HashMap();
+    protected HashMap<String,Group> groups = new HashMap<String,Group>();
 
 
     /**
@@ -111,7 +114,7 @@ public class MemoryUserDatabase implements UserDatabase {
 
 
     /**
-     * The relative or absolute pathname ot the file in which we write
+     * The relative or absolute pathname of the file in which we write
      * our new information prior to renaming.
      */
     protected String pathnameNew = pathname + ".new";
@@ -126,13 +129,13 @@ public class MemoryUserDatabase implements UserDatabase {
      * The set of {@link Role}s defined in this database, keyed by
      * role name.
      */
-    protected HashMap roles = new HashMap();
+    protected HashMap<String,Role> roles = new HashMap<String,Role>();
 
 
     /**
      * The string manager for this package.
      */
-    private static StringManager sm =
+    private static final StringManager sm =
         StringManager.getManager(Constants.Package);
 
 
@@ -140,7 +143,7 @@ public class MemoryUserDatabase implements UserDatabase {
      * The set of {@link User}s defined in this database, keyed by
      * user name.
      */
-    protected HashMap users = new HashMap();
+    protected HashMap<String,User> users = new HashMap<String,User>();
 
 
     // ------------------------------------------------------------- Properties
@@ -149,7 +152,8 @@ public class MemoryUserDatabase implements UserDatabase {
     /**
      * Return the set of {@link Group}s defined in this user database.
      */
-    public Iterator getGroups() {
+    @Override
+    public Iterator<Group> getGroups() {
 
         synchronized (groups) {
             return (groups.values().iterator());
@@ -161,6 +165,7 @@ public class MemoryUserDatabase implements UserDatabase {
     /**
      * Return the unique global identifier of this user database.
      */
+    @Override
     public String getId() {
 
         return (this.id);
@@ -205,7 +210,7 @@ public class MemoryUserDatabase implements UserDatabase {
     /**
      * Setting the readonly status of the user database
      *
-     * @param pathname The new pathname
+     * @param readonly the new status
      */
     public void setReadonly(boolean readonly) {
 
@@ -217,7 +222,8 @@ public class MemoryUserDatabase implements UserDatabase {
     /**
      * Return the set of {@link Role}s defined in this user database.
      */
-    public Iterator getRoles() {
+    @Override
+    public Iterator<Role> getRoles() {
 
         synchronized (roles) {
             return (roles.values().iterator());
@@ -229,7 +235,8 @@ public class MemoryUserDatabase implements UserDatabase {
     /**
      * Return the set of {@link User}s defined in this user database.
      */
-    public Iterator getUsers() {
+    @Override
+    public Iterator<User> getUsers() {
 
         synchronized (users) {
             return (users.values().iterator());
@@ -247,6 +254,7 @@ public class MemoryUserDatabase implements UserDatabase {
      *
      * @exception Exception if any exception is thrown during closing
      */
+    @Override
     public void close() throws Exception {
 
         save();
@@ -267,8 +275,15 @@ public class MemoryUserDatabase implements UserDatabase {
      * @param groupname The group name of the new group (must be unique)
      * @param description The description of this group
      */
+    @Override
     public Group createGroup(String groupname, String description) {
 
+        if (groupname == null || groupname.length() == 0) {
+            String msg = sm.getString("memoryUserDatabase.nullGroup");
+            log.warn(msg);
+            throw new IllegalArgumentException(msg);
+        }
+
         MemoryGroup group = new MemoryGroup(this, groupname, description);
         synchronized (groups) {
             groups.put(group.getGroupname(), group);
@@ -284,8 +299,15 @@ public class MemoryUserDatabase implements UserDatabase {
      * @param rolename The role name of the new group (must be unique)
      * @param description The description of this group
      */
+    @Override
     public Role createRole(String rolename, String description) {
 
+        if (rolename == null || rolename.length() == 0) {
+            String msg = sm.getString("memoryUserDatabase.nullRole");
+            log.warn(msg);
+            throw new IllegalArgumentException(msg);
+        }
+
         MemoryRole role = new MemoryRole(this, rolename, description);
         synchronized (roles) {
             roles.put(role.getRolename(), role);
@@ -302,15 +324,21 @@ public class MemoryUserDatabase implements UserDatabase {
      * @param password The logon password of the new user
      * @param fullName The full name of the new user
      */
+    @Override
     public User createUser(String username, String password,
                            String fullName) {
 
+        if (username == null || username.length() == 0) {
+            String msg = sm.getString("memoryUserDatabase.nullUser");
+            log.warn(msg);
+            throw new IllegalArgumentException(msg);
+        }
+
         MemoryUser user = new MemoryUser(this, username, password, fullName);
         synchronized (users) {
             users.put(user.getUsername(), user);
         }
         return (user);
-
     }
 
 
@@ -320,10 +348,11 @@ public class MemoryUserDatabase implements UserDatabase {
      *
      * @param groupname Name of the group to return
      */
+    @Override
     public Group findGroup(String groupname) {
 
         synchronized (groups) {
-            return ((Group) groups.get(groupname));
+            return groups.get(groupname);
         }
 
     }
@@ -335,10 +364,11 @@ public class MemoryUserDatabase implements UserDatabase {
      *
      * @param rolename Name of the role to return
      */
+    @Override
     public Role findRole(String rolename) {
 
         synchronized (roles) {
-            return ((Role) roles.get(rolename));
+            return roles.get(rolename);
         }
 
     }
@@ -350,10 +380,11 @@ public class MemoryUserDatabase implements UserDatabase {
      *
      * @param username Name of the user to return
      */
+    @Override
     public User findUser(String username) {
 
         synchronized (users) {
-            return ((User) users.get(username));
+            return users.get(username);
         }
 
     }
@@ -364,6 +395,7 @@ public class MemoryUserDatabase implements UserDatabase {
      *
      * @exception Exception if any exception is thrown during opening
      */
+    @Override
     public void open() throws Exception {
 
         synchronized (groups) {
@@ -377,7 +409,7 @@ public class MemoryUserDatabase implements UserDatabase {
                 // Construct a reader for the XML input file (if it exists)
                 File file = new File(pathname);
                 if (!file.isAbsolute()) {
-                    file = new File(System.getProperty("catalina.base"),
+                    file = new File(System.getProperty(Globals.CATALINA_BASE_PROP),
                                     pathname);
                 }
                 if (!file.exists()) {
@@ -396,13 +428,13 @@ public class MemoryUserDatabase implements UserDatabase {
                 }
                 digester.addFactoryCreate
                     ("tomcat-users/group",
-                     new MemoryGroupCreationFactory(this));
+                     new MemoryGroupCreationFactory(this), true);
                 digester.addFactoryCreate
                     ("tomcat-users/role",
-                     new MemoryRoleCreationFactory(this));
+                     new MemoryRoleCreationFactory(this), true);
                 digester.addFactoryCreate
                     ("tomcat-users/user",
-                     new MemoryUserCreationFactory(this));
+                     new MemoryUserCreationFactory(this), true);
 
                 // Parse the XML input file to load this database
                 try {
@@ -412,7 +444,7 @@ public class MemoryUserDatabase implements UserDatabase {
                     try {
                         fis.close();
                     } catch (Throwable t) {
-                        ;
+                        ExceptionUtils.handleThrowable(t);
                     }
                     throw e;
                 }
@@ -428,12 +460,13 @@ public class MemoryUserDatabase implements UserDatabase {
      *
      * @param group The group to be removed
      */
+    @Override
     public void removeGroup(Group group) {
 
         synchronized (groups) {
-            Iterator users = getUsers();
+            Iterator<User> users = getUsers();
             while (users.hasNext()) {
-                User user = (User) users.next();
+                User user = users.next();
                 user.removeGroup(group);
             }
             groups.remove(group.getGroupname());
@@ -447,17 +480,18 @@ public class MemoryUserDatabase implements UserDatabase {
      *
      * @param role The role to be removed
      */
+    @Override
     public void removeRole(Role role) {
 
         synchronized (roles) {
-            Iterator groups = getGroups();
+            Iterator<Group> groups = getGroups();
             while (groups.hasNext()) {
-                Group group = (Group) groups.next();
+                Group group = groups.next();
                 group.removeRole(role);
             }
-            Iterator users = getUsers();
+            Iterator<User> users = getUsers();
             while (users.hasNext()) {
-                User user = (User) users.next();
+                User user = users.next();
                 user.removeRole(role);
             }
             roles.remove(role.getRolename());
@@ -471,6 +505,7 @@ public class MemoryUserDatabase implements UserDatabase {
      *
      * @param user The user to be removed
      */
+    @Override
     public void removeUser(User user) {
 
         synchronized (users) {
@@ -489,7 +524,7 @@ public class MemoryUserDatabase implements UserDatabase {
 
         File file = new File(pathname);
         if (!file.isAbsolute()) {
-            file = new File(System.getProperty("catalina.base"),
+            file = new File(System.getProperty(Globals.CATALINA_BASE_PROP),
                             pathname);
         }
         File dir = file.getParentFile();
@@ -504,6 +539,7 @@ public class MemoryUserDatabase implements UserDatabase {
      *
      * @exception Exception if any exception is thrown during saving
      */
+    @Override
     public void save() throws Exception {
 
         if (getReadonly()) {
@@ -520,7 +556,7 @@ public class MemoryUserDatabase implements UserDatabase {
         File fileNew = new File(pathnameNew);
         if (!fileNew.isAbsolute()) {
             fileNew =
-                new File(System.getProperty("catalina.base"), pathnameNew);
+                new File(System.getProperty(Globals.CATALINA_BASE_PROP), pathnameNew);
         }
         PrintWriter writer = null;
         try {
@@ -535,7 +571,7 @@ public class MemoryUserDatabase implements UserDatabase {
             writer.println("<tomcat-users>");
 
             // Print entries for each defined role, group, and user
-            Iterator values = null;
+            Iterator<?> values = null;
             values = getRoles();
             while (values.hasNext()) {
                 writer.print("  ");
@@ -549,7 +585,7 @@ public class MemoryUserDatabase implements UserDatabase {
             values = getUsers();
             while (values.hasNext()) {
                 writer.print("  ");
-                writer.println(((MemoryUser) values.next()).toXml());
+                writer.println(values.next());
             }
 
             // Print the file epilog
@@ -576,13 +612,13 @@ public class MemoryUserDatabase implements UserDatabase {
         File fileOld = new File(pathnameOld);
         if (!fileOld.isAbsolute()) {
             fileOld =
-                new File(System.getProperty("catalina.base"), pathnameOld);
+                new File(System.getProperty(Globals.CATALINA_BASE_PROP), pathnameOld);
         }
         fileOld.delete();
         File fileOrig = new File(pathname);
         if (!fileOrig.isAbsolute()) {
             fileOrig =
-                new File(System.getProperty("catalina.base"), pathname);
+                new File(System.getProperty(Globals.CATALINA_BASE_PROP), pathname);
         }
         if (fileOrig.exists()) {
             fileOld.delete();
@@ -608,9 +644,10 @@ public class MemoryUserDatabase implements UserDatabase {
     /**
      * Return a String representation of this UserDatabase.
      */
+    @Override
     public String toString() {
 
-        StringBuffer sb = new StringBuffer("MemoryUserDatabase[id=");
+        StringBuilder sb = new StringBuilder("MemoryUserDatabase[id=");
         sb.append(this.id);
         sb.append(",pathname=");
         sb.append(pathname);
@@ -646,12 +683,13 @@ public class MemoryUserDatabase implements UserDatabase {
 /**
  * Digester object creation factory for group instances.
  */
-class MemoryGroupCreationFactory implements ObjectCreationFactory {
+class MemoryGroupCreationFactory extends AbstractObjectCreationFactory {
 
     public MemoryGroupCreationFactory(MemoryUserDatabase database) {
         this.database = database;
     }
 
+    @Override
     public Object createObject(Attributes attributes) {
         String groupname = attributes.getValue("groupname");
         if (groupname == null) {
@@ -684,29 +722,19 @@ class MemoryGroupCreationFactory implements ObjectCreationFactory {
     }
 
     private MemoryUserDatabase database = null;
-
-    private Digester digester = null;
-
-    public Digester getDigester() {
-        return (this.digester);
-    }
-
-    public void setDigester(Digester digester) {
-        this.digester = digester;
-    }
-
 }
 
 
 /**
  * Digester object creation factory for role instances.
  */
-class MemoryRoleCreationFactory implements ObjectCreationFactory {
+class MemoryRoleCreationFactory extends AbstractObjectCreationFactory {
 
     public MemoryRoleCreationFactory(MemoryUserDatabase database) {
         this.database = database;
     }
 
+    @Override
     public Object createObject(Attributes attributes) {
         String rolename = attributes.getValue("rolename");
         if (rolename == null) {
@@ -718,29 +746,19 @@ class MemoryRoleCreationFactory implements ObjectCreationFactory {
     }
 
     private MemoryUserDatabase database = null;
-
-    private Digester digester = null;
-
-    public Digester getDigester() {
-        return (this.digester);
-    }
-
-    public void setDigester(Digester digester) {
-        this.digester = digester;
-    }
-
 }
 
 
 /**
  * Digester object creation factory for user instances.
  */
-class MemoryUserCreationFactory implements ObjectCreationFactory {
+class MemoryUserCreationFactory extends AbstractObjectCreationFactory {
 
     public MemoryUserCreationFactory(MemoryUserDatabase database) {
         this.database = database;
     }
 
+    @Override
     public Object createObject(Attributes attributes) {
         String username = attributes.getValue("username");
         if (username == null) {
@@ -798,15 +816,4 @@ class MemoryUserCreationFactory implements ObjectCreationFactory {
     }
 
     private MemoryUserDatabase database = null;
-
-    private Digester digester = null;
-
-    public Digester getDigester() {
-        return (this.digester);
-    }
-
-    public void setDigester(Digester digester) {
-        this.digester = digester;
-    }
-
 }
