@@ -26,11 +26,9 @@ import org.apache.coyote.InputBuffer;
 import org.apache.coyote.Request;
 import org.apache.tomcat.util.buf.ByteChunk;
 import org.apache.tomcat.util.buf.MessageBytes;
-import org.apache.tomcat.util.http.MimeHeaders;
 import org.apache.tomcat.util.net.NioChannel;
-import org.apache.tomcat.util.net.NioSelectorPool;
-import org.apache.tomcat.util.res.StringManager;
 import org.apache.tomcat.util.net.NioEndpoint;
+import org.apache.tomcat.util.net.NioSelectorPool;
 
 /**
  * Implementation of InputBuffer which provides HTTP request header parsing as
@@ -39,7 +37,7 @@ import org.apache.tomcat.util.net.NioEndpoint;
  * @author <a href="mailto:remm@apache.org">Remy Maucherat</a>
  * @author Filip Hanik
  */
-public class InternalNioInputBuffer implements InputBuffer {
+public class InternalNioInputBuffer extends AbstractInputBuffer {
 
     /**
      * Logger.
@@ -82,37 +80,10 @@ public class InternalNioInputBuffer implements InputBuffer {
 
     }
 
-
-    // -------------------------------------------------------------- Variables
-
-
-    /**
-     * The string manager for this package.
-     */
-    protected static StringManager sm =
-        StringManager.getManager(Constants.Package);
-
-
-    // ----------------------------------------------------- Instance Variables
-
-
-    /**
-     * Associated Coyote request.
-     */
-    protected Request request;
-
-
-    /**
-     * Headers of the associated request.
-     */
-    protected MimeHeaders headers;
-
-
     /**
      * Parsing state - used for non blocking parsing so that
      * when more data arrives, we can pick up where we left off.
      */
-    protected boolean parsingHeader;
     protected boolean parsingRequestLine;
     protected int parsingRequestLinePhase = 0;
     protected boolean parsingRequestLineEol = false;
@@ -120,39 +91,6 @@ public class InternalNioInputBuffer implements InputBuffer {
     protected int parsingRequestLineQPos = -1;
     protected HeaderParsePosition headerParsePos;
 
-
-    /**
-     * Swallow input ? (in the case of an expectation)
-     */
-    protected boolean swallowInput;
-
-
-    /**
-     * Pointer to the current read buffer.
-     */
-    protected byte[] buf;
-
-
-    /**
-     * Last valid byte.
-     */
-    protected int lastValid;
-
-
-    /**
-     * Position in the buffer.
-     */
-    protected int pos;
-
-
-    /**
-     * Pos of the end of the header in the buffer, which is also the
-     * start of the body.
-     */
-    protected int end;
-
-
-
     /**
      * Underlying socket.
      */
@@ -162,31 +100,6 @@ public class InternalNioInputBuffer implements InputBuffer {
      * Selector pool, for blocking reads and blocking writes
      */
     protected NioSelectorPool pool;
-    
-
-    /**
-     * Underlying input buffer.
-     */
-    protected InputBuffer inputStreamInputBuffer;
-
-
-    /**
-     * Filter library.
-     * Note: Filter[0] is always the "chunked" filter.
-     */
-    protected InputFilter[] filterLibrary;
-
-
-    /**
-     * Active filters (in order).
-     */
-    protected InputFilter[] activeFilters;
-
-
-    /**
-     * Index of the last active filter.
-     */
-    protected int lastActiveFilter;
 
 
     /**
@@ -243,84 +156,7 @@ public class InternalNioInputBuffer implements InputBuffer {
     }
 
 
-    /**
-     * Add an input filter to the filter library.
-     */
-    public void addFilter(InputFilter filter) {
-
-        InputFilter[] newFilterLibrary = 
-            new InputFilter[filterLibrary.length + 1];
-        for (int i = 0; i < filterLibrary.length; i++) {
-            newFilterLibrary[i] = filterLibrary[i];
-        }
-        newFilterLibrary[filterLibrary.length] = filter;
-        filterLibrary = newFilterLibrary;
-
-        activeFilters = new InputFilter[filterLibrary.length];
-
-    }
-
-
-    /**
-     * Get filters.
-     */
-    public InputFilter[] getFilters() {
-
-        return filterLibrary;
-
-    }
-
-
-    /**
-     * Clear filters.
-     */
-    public void clearFilters() {
-
-        filterLibrary = new InputFilter[0];
-        lastActiveFilter = -1;
-
-    }
-
-
-    /**
-     * Add an input filter to the filter library.
-     */
-    public void addActiveFilter(InputFilter filter) {
-
-        if (lastActiveFilter == -1) {
-            filter.setBuffer(inputStreamInputBuffer);
-        } else {
-            for (int i = 0; i <= lastActiveFilter; i++) {
-                if (activeFilters[i] == filter)
-                    return;
-            }
-            filter.setBuffer(activeFilters[lastActiveFilter]);
-        }
-
-        activeFilters[++lastActiveFilter] = filter;
-
-        filter.setRequest(request);
-
-    }
-
-
-    /**
-     * Set the swallow input flag.
-     */
-    public void setSwallowInput(boolean swallowInput) {
-        this.swallowInput = swallowInput;
-    }
-
     // --------------------------------------------------------- Public Methods
-    /**
-     * Returns true if there are bytes available from the socket layer
-     * @return boolean
-     * @throws IOException
-     */
-    public boolean isReadable() throws IOException {
-        return (pos < lastValid) || (nbRead()>0);
-    }
-    
     /**
      * Issues a non blocking read
      * @return int
@@ -334,20 +170,15 @@ public class InternalNioInputBuffer implements InputBuffer {
      * Recycle the input buffer. This should be called when closing the 
      * connection.
      */
+    @Override
     public void recycle() {
         // Recycle filters
         for (int i = 0; i <= lastActiveFilter; i++) {
             activeFilters[i].recycle();
         }
-
-        // Recycle Request object
-        request.recycle();
-
+        // This must be after filters since it resets the lastFilterIndex
+        super.recycle();
         socket = null;
-        lastValid = 0;
-        pos = 0;
-        lastActiveFilter = -1;
-        parsingHeader = true;
         headerParsePos = HeaderParsePosition.HEADER_START;
         parsingRequestLine = true;
         parsingRequestLinePhase = 0;
@@ -355,8 +186,6 @@ public class InternalNioInputBuffer implements InputBuffer {
         parsingRequestLineStart = 0;
         parsingRequestLineQPos = -1;
         headerData.recycle();
-        swallowInput = true;
-
     }
 
 
@@ -366,33 +195,9 @@ public class InternalNioInputBuffer implements InputBuffer {
      * consumed. This method only resets all the pointers so that we are ready
      * to parse the next HTTP request.
      */
+    @Override
     public void nextRequest() {
-
-        // Recycle Request object
-        request.recycle();
-
-        // Copy leftover bytes to the beginning of the buffer
-        if (lastValid - pos > 0) {
-            int npos = 0;
-            int opos = pos;
-            while (lastValid - opos > opos - npos) {
-                System.arraycopy(buf, opos, buf, npos, opos - npos);
-                npos += pos;
-                opos += pos;
-            }
-            System.arraycopy(buf, opos, buf, npos, lastValid - opos);
-        }
-
-        // Recycle filters
-        for (int i = 0; i <= lastActiveFilter; i++) {
-            activeFilters[i].recycle();
-        }
-
-        // Reset pointers
-        lastValid = lastValid - pos;
-        pos = 0;
-        lastActiveFilter = -1;
-        parsingHeader = true;
+        super.nextRequest();
         headerParsePos = HeaderParsePosition.HEADER_START;
         parsingRequestLine = true;
         parsingRequestLinePhase = 0;
@@ -400,38 +205,20 @@ public class InternalNioInputBuffer implements InputBuffer {
         parsingRequestLineStart = 0;
         parsingRequestLineQPos = -1;
         headerData.recycle();
-        swallowInput = true;
-
     }
 
-
-    /**
-     * End request (consumes leftover bytes).
-     * 
-     * @throws IOException an undelying I/O error occured
-     */
-    public void endRequest()
-        throws IOException {
-
-        if (swallowInput && (lastActiveFilter != -1)) {
-            int extraBytes = (int) activeFilters[lastActiveFilter].end();
-            pos = pos - extraBytes;
-        }
-
-    }
-
-
     /**
      * Read the request line. This function is meant to be used during the 
      * HTTP request header parsing. Do NOT attempt to read the request body 
      * using it.
      *
      * @throws IOException If an exception occurs during the underlying socket
-     * read operations, or if the given buffer is not big enough to accomodate
+     * read operations, or if the given buffer is not big enough to accommodate
      * the whole line.
      * @return true if data is properly fed; false if no data is available 
      * immediately and thread should be freed
      */
+    @Override
     public boolean parseRequestLine(boolean useAvailableDataOnly)
         throws IOException {
 
@@ -467,6 +254,11 @@ public class InternalNioInputBuffer implements InputBuffer {
             skipBlankLinesBytes = pos;
             parsingRequestLineStart = pos;
             parsingRequestLinePhase = 2;
+            if (log.isDebugEnabled()) {
+                log.debug("Received ["
+                        + new String(buf, pos, lastValid - pos, "ISO-8859-1")
+                        + "]");
+            }
         }
         if ( parsingRequestLinePhase == 2 ) {
             //
@@ -619,7 +411,6 @@ public class InternalNioInputBuffer implements InputBuffer {
             byte[] tmp = new byte[newsize];
             System.arraycopy(buf,0,tmp,0,buf.length);
             buf = tmp;
-            tmp = null;
         }
     }
     
@@ -631,12 +422,17 @@ public class InternalNioInputBuffer implements InputBuffer {
      * @throws IOException if a socket exception occurs
      * @throws EOFException if end of stream is reached
      */
+    
     private int readSocket(boolean timeout, boolean block) throws IOException {
         int nRead = 0;
         socket.getBufHandler().getReadBuffer().clear();
         if ( block ) {
             Selector selector = null;
-            try { selector = getSelectorPool().get(); }catch ( IOException x ) {}
+            try {
+                selector = getSelectorPool().get();
+            } catch ( IOException x ) {
+                // Ignore
+            }
             try {
                 NioEndpoint.KeyAttachment att = (NioEndpoint.KeyAttachment)socket.getAttachment(false);
                 if ( att == null ) throw new IOException("Key must be cancelled.");
@@ -667,12 +463,17 @@ public class InternalNioInputBuffer implements InputBuffer {
     /**
      * Parse the HTTP headers.
      */
+    @Override
     public boolean parseHeaders()
         throws IOException {
         HeaderParseStatus status = HeaderParseStatus.HAVE_MORE_HEADERS;
         
         do {
             status = parseHeader();
+        } while ( status == HeaderParseStatus.HAVE_MORE_HEADERS );
+        if (status == HeaderParseStatus.DONE) {
+            parsingHeader = false;
+            end = pos;
             // Checking that
             // (1) Headers plus request line size does not exceed its limit
             // (2) There are enough bytes to avoid expanding the buffer when
@@ -681,15 +482,11 @@ public class InternalNioInputBuffer implements InputBuffer {
             // limitation to enforce the meaning of headerBufferSize
             // From the way how buf is allocated and how blank lines are being
             // read, it should be enough to check (1) only.
-            if (pos - skipBlankLinesBytes > headerBufferSize
-                    || buf.length - pos < socketReadBufferSize) {
+            if (end - skipBlankLinesBytes > headerBufferSize
+                    || buf.length - end < socketReadBufferSize) {
                 throw new IllegalArgumentException(
                         sm.getString("iib.requestheadertoolarge.error"));
             }
-        } while ( status == HeaderParseStatus.HAVE_MORE_HEADERS );
-        if (status == HeaderParseStatus.DONE) {
-            parsingHeader = false;
-            end = pos;
             return true;
         } else {
             return false;
@@ -746,8 +543,7 @@ public class InternalNioInputBuffer implements InputBuffer {
         // Reading the header name
         // Header name is always US-ASCII
         //
-        
-        
+
 
         while (headerParsePos == HeaderParsePosition.HEADER_NAME) {
 
@@ -822,6 +618,7 @@ public class InternalNioInputBuffer implements InputBuffer {
                     }
 
                     if (buf[pos] == Constants.CR) {
+                        // Skip
                     } else if (buf[pos] == Constants.LF) {
                         eol = true;
                     } else if (buf[pos] == Constants.SP) {
@@ -901,32 +698,20 @@ public class InternalNioInputBuffer implements InputBuffer {
     }
 
 
-    // ---------------------------------------------------- InputBuffer Methods
-
-
-    /**
-     * Read some bytes.
-     */
-    public int doRead(ByteChunk chunk, Request req) 
-        throws IOException {
-
-        if (lastActiveFilter == -1)
-            return inputStreamInputBuffer.doRead(chunk, req);
-        else
-            return activeFilters[lastActiveFilter].doRead(chunk,req);
-
-    }
-
-
     // ------------------------------------------------------ Protected Methods
 
     /**
-     * Fill the internal buffer using data from the undelying input stream.
+     * Fill the internal buffer using data from the underlying input stream.
      * 
      * @return false if at end of stream
      */
-    protected boolean fill(boolean timeout, boolean block)
-        throws IOException, EOFException {
+    @Override
+    protected boolean fill(boolean block) throws IOException, EOFException {
+        return fill(true,block);
+    }
+
+    protected boolean fill(boolean timeout, boolean block) throws IOException, EOFException {
+        
 
         boolean read = false;
 
@@ -962,6 +747,7 @@ public class InternalNioInputBuffer implements InputBuffer {
         /**
          * Read bytes into the specified chunk.
          */
+        @Override
         public int doRead(ByteChunk chunk, Request req ) 
             throws IOException {
 
@@ -982,4 +768,9 @@ public class InternalNioInputBuffer implements InputBuffer {
     }
 
 
+    public int getParsingRequestLinePhase() {
+        return parsingRequestLinePhase;
+    }
+
+
 }
