@@ -34,14 +34,18 @@ import java.net.MalformedURLException;
 import java.net.URL;
 import java.net.URLClassLoader;
 import java.util.ArrayList;
+import java.util.HashMap;
+import java.util.HashSet;
 import java.util.Iterator;
 import java.util.List;
 import java.util.Map;
-import java.util.HashMap;
+import java.util.Set;
 import java.util.Stack;
 import java.util.StringTokenizer;
 import java.util.Vector;
 
+import javax.servlet.jsp.tagext.TagLibraryInfo;
+
 import org.apache.jasper.compiler.Compiler;
 import org.apache.jasper.compiler.JspConfig;
 import org.apache.jasper.compiler.JspRuntimeContext;
@@ -51,7 +55,6 @@ import org.apache.jasper.compiler.TldLocationsCache;
 import org.apache.jasper.servlet.JspCServletContext;
 import org.apache.juli.logging.Log;
 import org.apache.juli.logging.LogFactory;
-
 import org.apache.tools.ant.AntClassLoader;
 import org.apache.tools.ant.Project;
 import org.apache.tools.ant.util.FileUtils;
@@ -94,7 +97,7 @@ public class JspC implements Options {
             "clsid:8AD9C840-044E-11D1-B3E9-00805F499D93";
 
     // Logger
-    protected static Log log = LogFactory.getLog(JspC.class);
+    private static final Log log = LogFactory.getLog(JspC.class);
 
     protected static final String SWITCH_VERBOSE = "-v";
     protected static final String SWITCH_HELP = "-help";
@@ -122,22 +125,32 @@ public class JspC implements Options {
     protected static final String SWITCH_ENCODING = "-javaEncoding";
     protected static final String SWITCH_SMAP = "-smap";
     protected static final String SWITCH_DUMP_SMAP = "-dumpsmap";
-
     protected static final String SHOW_SUCCESS ="-s";
     protected static final String LIST_ERRORS = "-l";
     protected static final int INC_WEBXML = 10;
     protected static final int ALL_WEBXML = 20;
     protected static final int DEFAULT_DIE_LEVEL = 1;
     protected static final int NO_DIE_LEVEL = 0;
+    protected static final Set<String> insertBefore = new HashSet<String>();
+    
+    static {
+        insertBefore.add("</web-app>");
+        insertBefore.add("<servlet-mapping>");
+        insertBefore.add("<session-config>");
+        insertBefore.add("<mime-mapping>");
+        insertBefore.add("<welcome-file-list>");
+        insertBefore.add("<error-page>");
+        insertBefore.add("<taglib>");
+        insertBefore.add("<resource-env-ref>");
+        insertBefore.add("<resource-ref>");
+        insertBefore.add("<security-constraint>");
+        insertBefore.add("<login-config>");
+        insertBefore.add("<security-role>");
+        insertBefore.add("<env-entry>");
+        insertBefore.add("<ejb-ref>");
+        insertBefore.add("<ejb-local-ref>");
+    }
 
-    protected static final String[] insertBefore =
-    { "</web-app>", "<servlet-mapping>", "<session-config>",
-      "<mime-mapping>", "<welcome-file-list>", "<error-page>", "<taglib>",
-      "<resource-env-ref>", "<resource-ref>", "<security-constraint>",
-      "<login-config>", "<security-role>", "<env-entry>", "<ejb-ref>",
-      "<ejb-local-ref>" };
-
-    protected static int die;
     protected String classPath = null;
     protected URLClassLoader loader = null;
     protected boolean trimSpaces = false;
@@ -158,12 +171,13 @@ public class JspC implements Options {
     protected boolean smapSuppressed = true;
     protected boolean smapDumped = false;
     protected boolean caching = true;
-    protected Map cache = new HashMap();
+    protected Map<String, TagLibraryInfo> cache =
+        new HashMap<String, TagLibraryInfo>();
 
     protected String compiler = null;
 
-    protected String compilerTargetVM = "1.5";
-    protected String compilerSourceVM = "1.5";
+    protected String compilerTargetVM = "1.6";
+    protected String compilerSourceVM = "1.6";
 
     protected boolean classDebugInfo = true;
 
@@ -177,12 +191,12 @@ public class JspC implements Options {
      * The file extensions to be handled as JSP files.
      * Default list is .jsp and .jspx.
      */
-    protected List extensions;
+    protected List<String> extensions;
 
     /**
      * The pages.
      */
-    protected List pages = new Vector();
+    protected List<String> pages = new Vector<String>();
 
     /**
      * Needs better documentation, this data member does.
@@ -236,8 +250,9 @@ public class JspC implements Options {
         if (arg.length == 0) {
             System.out.println(Localizer.getMessage("jspc.usage"));
         } else {
+            JspC jspc = null;
             try {
-                JspC jspc = new JspC();
+                jspc = new JspC();
                 jspc.setArgs(arg);
                 if (jspc.helpNeeded) {
                     System.out.println(Localizer.getMessage("jspc.usage"));
@@ -246,8 +261,8 @@ public class JspC implements Options {
                 }
             } catch (JasperException je) {
                 System.err.println(je);
-                if (die != NO_DIE_LEVEL) {
-                    System.exit(die);
+                if (jspc != null && jspc.dieLevel != NO_DIE_LEVEL) {
+                    System.exit(jspc.dieLevel);
                 }
             }
         }
@@ -264,7 +279,6 @@ public class JspC implements Options {
         String tok;
 
         dieLevel = NO_DIE_LEVEL;
-        die = dieLevel;
 
         while ((tok = nextArg()) != null) {
             if (tok.equals(SWITCH_VERBOSE)) {
@@ -326,7 +340,6 @@ public class JspC implements Options {
                 } catch (NumberFormatException nfe) {
                     dieLevel = DEFAULT_DIE_LEVEL;
                 }
-                die = dieLevel;
             } else if (tok.equals(SWITCH_HELP)) {
                 helpNeeded = true;
             } else if (tok.equals(SWITCH_POOLING)) {
@@ -373,6 +386,7 @@ public class JspC implements Options {
      * In JspC this always returns <code>true</code>.
      * {@inheritDoc}
      */
+    @Override
     public boolean getKeepGenerated() {
         // isn't this why we are running jspc?
         return true;
@@ -381,6 +395,7 @@ public class JspC implements Options {
     /**
      * {@inheritDoc}
      */
+    @Override
     public boolean getTrimSpaces() {
         return trimSpaces;
     }
@@ -395,6 +410,7 @@ public class JspC implements Options {
     /**
      * {@inheritDoc}
      */
+    @Override
     public boolean isPoolingEnabled() {
         return poolingEnabled;
     }
@@ -409,6 +425,7 @@ public class JspC implements Options {
     /**
      * {@inheritDoc}
      */
+    @Override
     public boolean isXpoweredBy() {
         return xpoweredBy;
     }
@@ -424,13 +441,25 @@ public class JspC implements Options {
      * In JspC this always returns <code>true</code>.
      * {@inheritDoc}
      */
+    @Override
     public boolean getDisplaySourceFragment() {
         return true;
     }
 
+    @Override
+    public int getMaxLoadedJsps() {
+        return -1;
+    }
+
+    @Override
+    public int getJspIdleTimeout() {
+        return -1;
+    }
+
     /**
      * {@inheritDoc}
      */
+    @Override
     public boolean getErrorOnUseBeanInvalidClassAttribute() {
         return errorOnUseBeanInvalidClassAttribute;
     }
@@ -443,38 +472,14 @@ public class JspC implements Options {
         errorOnUseBeanInvalidClassAttribute = b;
     }
 
-    /**
-     * @deprecated Removed in Tomcat 7
-     */
-    @Deprecated
-    public int getTagPoolSize() {
-        return Constants.MAX_POOL_SIZE;
-    }
-
     /**
      * {@inheritDoc}
      */
+    @Override
     public boolean getMappedFile() {
         return mappedFile;
     }
 
-    /**
-     * @deprecated Removed in Tomcat 7
-     */
-    @Deprecated
-    public Object getProtectionDomain() {
-        // Off-line compiler, no need for security manager
-        return null;
-    }
-
-    /**
-     * @deprecated
-     */
-    @Deprecated
-    public boolean getSendErrorToClient() {
-        return true;
-    }
-
     /**
      * Sets the option to include debug information in compiled class.
      */
@@ -485,6 +490,7 @@ public class JspC implements Options {
     /**
      * {@inheritDoc}
      */
+    @Override
     public boolean getClassDebugInfo() {
         // compile with debug info
         return classDebugInfo;
@@ -493,6 +499,7 @@ public class JspC implements Options {
     /**
      * {@inheritDoc}
      */
+    @Override
     public boolean isCaching() {
         return caching;
     }
@@ -509,7 +516,8 @@ public class JspC implements Options {
     /**
      * {@inheritDoc}
      */
-    public Map getCache() {
+    @Override
+    public Map<String, TagLibraryInfo> getCache() {
         return cache;
     }
 
@@ -517,6 +525,7 @@ public class JspC implements Options {
      * In JspC this always returns <code>0</code>.
      * {@inheritDoc}
      */
+    @Override
     public int getCheckInterval() {
         return 0;
     }
@@ -525,6 +534,7 @@ public class JspC implements Options {
      * In JspC this always returns <code>0</code>.
      * {@inheritDoc}
      */
+    @Override
     public int getModificationTestInterval() {
         return 0;
     }
@@ -534,6 +544,7 @@ public class JspC implements Options {
      * In JspC this always returns <code>false</code>.
      * {@inheritDoc}
      */
+    @Override
     public boolean getRecompileOnFail() {
         return false;
     }
@@ -543,6 +554,7 @@ public class JspC implements Options {
      * In JspC this always returns <code>false</code>.
      * {@inheritDoc}
      */
+    @Override
     public boolean getDevelopment() {
         return false;
     }
@@ -550,6 +562,7 @@ public class JspC implements Options {
     /**
      * {@inheritDoc}
      */
+    @Override
     public boolean isSmapSuppressed() {
         return smapSuppressed;
     }
@@ -564,6 +577,7 @@ public class JspC implements Options {
     /**
      * {@inheritDoc}
      */
+    @Override
     public boolean isSmapDumped() {
         return smapDumped;
     }
@@ -592,6 +606,7 @@ public class JspC implements Options {
     /**
      * {@inheritDoc}
      */
+    @Override
     public boolean genStringAsCharArray() {
         return genStringAsCharArray;
     }
@@ -610,6 +625,7 @@ public class JspC implements Options {
     /**
      * {@inheritDoc}
      */
+    @Override
     public String getIeClassId() {
         return ieClassId;
     }
@@ -617,31 +633,15 @@ public class JspC implements Options {
     /**
      * {@inheritDoc}
      */
+    @Override
     public File getScratchDir() {
         return scratchDir;
     }
 
-    /**
-     * @deprecated Removed in Tomcat 7
-     */
-    @Deprecated
-    public Class getJspCompilerPlugin() {
-       // we don't compile, so this is meanlingless
-        return null;
-    }
-
-    /**
-     * @deprecated Removed in Tomcat 7
-     */
-    @Deprecated
-    public String getJspCompilerPath() {
-       // we don't compile, so this is meanlingless
-        return null;
-    }
-
     /**
      * {@inheritDoc}
      */
+    @Override
     public String getCompiler() {
         return compiler;
     }
@@ -658,6 +658,7 @@ public class JspC implements Options {
     /**
      * {@inheritDoc}
      */
+    @Override
     public String getCompilerClassName() {
         return null;
     }
@@ -665,6 +666,7 @@ public class JspC implements Options {
     /**
      * {@inheritDoc}
      */
+    @Override
     public String getCompilerTargetVM() {
         return compilerTargetVM;
     }
@@ -681,7 +683,8 @@ public class JspC implements Options {
     /**
      * {@inheritDoc}
      */
-     public String getCompilerSourceVM() {
+     @Override
+    public String getCompilerSourceVM() {
          return compilerSourceVM;
      }
 
@@ -697,6 +700,7 @@ public class JspC implements Options {
     /**
      * {@inheritDoc}
      */
+    @Override
     public TldLocationsCache getTldLocationsCache() {
         return tldLocationsCache;
     }
@@ -707,6 +711,7 @@ public class JspC implements Options {
      *
      * @return String The encoding
      */
+    @Override
     public String getJavaEncoding() {
         return javaEncoding;
     }
@@ -724,6 +729,7 @@ public class JspC implements Options {
     /**
      * {@inheritDoc}
      */
+    @Override
     public boolean getFork() {
         return false;
     }
@@ -731,6 +737,7 @@ public class JspC implements Options {
     /**
      * {@inheritDoc}
      */
+    @Override
     public String getClassPath() {
         if( classPath != null )
             return classPath;
@@ -751,7 +758,7 @@ public class JspC implements Options {
      *
      * @return The list of extensions
      */
-    public List getExtensions() {
+    public List<String> getExtensions() {
         return extensions;
     }
 
@@ -764,7 +771,7 @@ public class JspC implements Options {
     protected void addExtension(final String extension) {
         if(extension != null) {
             if(extensions == null) {
-                extensions = new Vector();
+                extensions = new Vector<String>();
             }
 
             extensions.add(extension);
@@ -795,8 +802,8 @@ public class JspC implements Options {
      * includes.
      */
     public void setUriroot( String s ) {
-        if( s==null ) {
-            uriRoot = s;
+        if (s == null) {
+            uriRoot = null;
             return;
         }
         try {
@@ -944,6 +951,7 @@ public class JspC implements Options {
     /**
      * {@inheritDoc}
      */
+    @Override
     public JspConfig getJspConfig() {
         return jspConfig;
     }
@@ -951,6 +959,7 @@ public class JspC implements Options {
     /**
      * {@inheritDoc}
      */
+    @Override
     public TagPluginManager getTagPluginManager() {
         return tagPluginManager;
     }
@@ -1024,16 +1033,7 @@ public class JspC implements Options {
         while (current > -1) {
             if (current == '<') {
                 String element = getElement(reader);
-                boolean found = false;
-                if (!inserted) {
-                    for (String before : insertBefore) {
-                        if (element.equals(before)) {
-                            found = true;
-                            break;
-                        }
-                    }
-                }
-                if (found) {
+                if (!inserted && insertBefore.contains(element)) {
                     // Insert generated content here
                     writer.println(insertStartMarker);
                     while (true) {
@@ -1095,12 +1095,15 @@ public class JspC implements Options {
         fis.close();
         fos.close();
 
-        webXml2.delete();
-        (new File(webxmlFile)).delete();
+        if(!webXml2.delete() && log.isDebugEnabled())
+            log.debug(Localizer.getMessage("jspc.delete.fail",
+                    webXml2.toString()));
+        
+        if (!(new File(webxmlFile)).delete() && log.isDebugEnabled())
+            log.debug(Localizer.getMessage("jspc.delete.fail", webxmlFile));
 
     }
-
-
+    
     /*
      * Assumes valid xml
      */
@@ -1136,7 +1139,6 @@ public class JspC implements Options {
         return result.toString();
     }
 
-
     protected void processFile(String file)
         throws JasperException
     {
@@ -1284,7 +1286,7 @@ public class JspC implements Options {
                     throw new JasperException(
                         Localizer.getMessage("jsp.error.jspc.missingTarget"));
                 }
-                String firstJsp = (String) pages.get( 0 );
+                String firstJsp = pages.get( 0 );
                 File firstJspF = new File( firstJsp );
                 if (!firstJspF.exists()) {
                     throw new JasperException(
@@ -1316,7 +1318,7 @@ public class JspC implements Options {
 
             initWebXml();
 
-            Iterator iter = pages.iterator();
+            Iterator<String> iter = pages.iterator();
             while (iter.hasNext()) {
                 String nextjsp = iter.next().toString();
                 File fjsp = new File(nextjsp);
@@ -1434,7 +1436,7 @@ public class JspC implements Options {
             context =new JspCServletContext
                 (new PrintWriter(System.out),
                  new URL("file:" + uriRoot.replace('\\','/') + '/'));
-            tldLocationsCache = new TldLocationsCache(context, true);
+            tldLocationsCache = TldLocationsCache.getInstance(context);
         } catch (MalformedURLException me) {
             System.out.println("**" + me);
         }
@@ -1469,7 +1471,7 @@ public class JspC implements Options {
             String path = tokenizer.nextToken();
             try {
                 File libFile = new File(path);
-                urls.add(libFile.toURL());
+                urls.add(libFile.toURI().toURL());
             } catch (IOException ioe) {
                 // Failing a toCanonicalPath on a file that
                 // exists() should be a JVM regression test,
@@ -1485,7 +1487,7 @@ public class JspC implements Options {
                 if (classes.exists()) {
                     classPath = classPath + File.pathSeparator
                         + classes.getCanonicalPath();
-                    urls.add(classes.getCanonicalFile().toURL());
+                    urls.add(classes.getCanonicalFile().toURI().toURL());
                 }
             } catch (IOException ioe) {
                 // failing a toCanonicalPath on a file that
@@ -1510,7 +1512,7 @@ public class JspC implements Options {
                         File libFile = new File(lib, libs[i]);
                         classPath = classPath + File.pathSeparator
                             + libFile.getAbsolutePath();
-                        urls.add(libFile.getAbsoluteFile().toURL());
+                        urls.add(libFile.getAbsoluteFile().toURI().toURL());
                     } catch (IOException ioe) {
                         // failing a toCanonicalPath on a file that
                         // exists() should be a JVM regression test,
@@ -1522,7 +1524,8 @@ public class JspC implements Options {
         }
 
         // What is this ??
-        urls.add(new File(clctxt.getRealPath("/")).getCanonicalFile().toURL());
+        urls.add(new File(
+                clctxt.getRealPath("/")).getCanonicalFile().toURI().toURL());
 
         URL urlsA[]=new URL[urls.size()];
         urls.toArray(urlsA);
@@ -1543,7 +1546,7 @@ public class JspC implements Options {
         try {
             if (f.exists()) {
                 f = new File(f.getAbsolutePath());
-                while (f != null) {
+                while (true) {
                     File g = new File(f, "WEB-INF");
                     if (g.exists() && g.isDirectory()) {
                         uriRoot = f.getCanonicalPath();
@@ -1566,7 +1569,7 @@ public class JspC implements Options {
                         f = new File(fParent);
                     }
 
-                    // If there is no acceptible candidate, uriRoot will
+                    // If there is no acceptable candidate, uriRoot will
                     // remain null to indicate to the CompilerContext to
                     // use the current working/user dir.
                 }
@@ -1597,9 +1600,9 @@ public class JspC implements Options {
      protected File resolveFile(final String s) {
          if(getProject() == null) {
              // Note FileUtils.getFileUtils replaces FileUtils.newFileUtils in Ant 1.6.3
-             return FileUtils.newFileUtils().resolveFile(null, s);
+             return FileUtils.getFileUtils().resolveFile(null, s);
          } else {
-             return FileUtils.newFileUtils().resolveFile(getProject().getBaseDir(), s);
+             return FileUtils.getFileUtils().resolveFile(getProject().getBaseDir(), s);
          }
      }
 
