@@ -33,6 +33,8 @@ import org.apache.jasper.JasperException;
 import org.apache.jasper.JspCompilationContext;
 import org.apache.jasper.Options;
 import org.apache.jasper.servlet.JspServletWrapper;
+import org.apache.juli.logging.Log;
+import org.apache.juli.logging.LogFactory;
 
 /**
  * Main JSP compiler class. This class uses Ant for compiling.
@@ -46,8 +48,7 @@ import org.apache.jasper.servlet.JspServletWrapper;
  */
 public abstract class Compiler {
     
-    protected org.apache.juli.logging.Log log = org.apache.juli.logging.LogFactory
-            .getLog(Compiler.class);
+    private final Log log = LogFactory.getLog(Compiler.class); // must not be static
 
     // ----------------------------------------------------- Instance Variables
 
@@ -139,6 +140,17 @@ public abstract class Compiler {
             pageInfo.setTrimDirectiveWhitespaces(JspUtil.booleanValue(jspProperty
                     .isTrimDirectiveWhitespaces()));
         }
+        // Default ContentType processing is deferred until after the page has
+        // been parsed
+        if (jspProperty.getBuffer() != null) {
+            pageInfo.setBufferValue(jspProperty.getBuffer(), null,
+                    errDispatcher);
+        }
+        if (jspProperty.isErrorOnUndeclaredNamespace() != null) {
+            pageInfo.setErrorOnUndeclaredNamespace(
+                    JspUtil.booleanValue(
+                            jspProperty.isErrorOnUndeclaredNamespace()));
+        }
         if (ctxt.isTagFile()) {
             try {
                 double libraryVersion = Double.parseDouble(ctxt.getTagInfo()
@@ -175,15 +187,21 @@ public abstract class Compiler {
              */ 
             // Parse the file
             ParserController parserCtl = new ParserController(ctxt, this);
-
+            
             // Pass 1 - the directives
             Node.Nodes directives =
                 parserCtl.parseDirectives(ctxt.getJspFile());
             Validator.validateDirectives(this, directives);
-
+            
             // Pass 2 - the whole translation unit
             pageNodes = parserCtl.parse(ctxt.getJspFile());
 
+            // Leave this until now since it can only be set once - bug 49726
+            if (pageInfo.getContentType() == null &&
+                    jspProperty.getDefaultContentType() != null) {
+                pageInfo.setContentType(jspProperty.getDefaultContentType());
+            }
+
             if (ctxt.isPrototypeMode()) {
                 // generate prototype .java file for the tag file
                 writer = setupContextWriter(javaFileName);
@@ -224,7 +242,7 @@ public abstract class Compiler {
             TextOptimizer.concatenate(this, pageNodes);
 
             // Generate static function mapper codes.
-            ELFunctionMapper.map(this, pageNodes);
+            ELFunctionMapper.map(pageNodes);
 
             // generate servlet .java file
             writer = setupContextWriter(javaFileName);
@@ -253,7 +271,14 @@ public abstract class Compiler {
                 }
             }
             // Remove the generated .java file
-            new File(javaFileName).delete();
+            File file = new File(javaFileName);
+            if (file.exists()) {
+                if (!file.delete()) {
+                    log.warn(Localizer.getMessage(
+                            "jsp.warning.compiler.javafile.delete.fail",
+                            file.getAbsolutePath()));
+                }
+            }
             throw e;
         } finally {
             if (writer != null) {
@@ -280,25 +305,25 @@ public abstract class Compiler {
         return smapStr;
     }
 
-	private ServletWriter setupContextWriter(String javaFileName)
-			throws FileNotFoundException, JasperException {
-		ServletWriter writer;
-		// Setup the ServletWriter
-		String javaEncoding = ctxt.getOptions().getJavaEncoding();
-		OutputStreamWriter osw = null;
+    private ServletWriter setupContextWriter(String javaFileName)
+            throws FileNotFoundException, JasperException {
+        ServletWriter writer;
+        // Setup the ServletWriter
+        String javaEncoding = ctxt.getOptions().getJavaEncoding();
+        OutputStreamWriter osw = null;
 
-		try {
-		    osw = new OutputStreamWriter(
-		            new FileOutputStream(javaFileName), javaEncoding);
-		} catch (UnsupportedEncodingException ex) {
-		    errDispatcher.jspError("jsp.error.needAlternateJavaEncoding",
-		            javaEncoding);
-		}
+        try {
+            osw = new OutputStreamWriter(
+                    new FileOutputStream(javaFileName), javaEncoding);
+        } catch (UnsupportedEncodingException ex) {
+            errDispatcher.jspError("jsp.error.needAlternateJavaEncoding",
+                    javaEncoding);
+        }
 
-		writer = new ServletWriter(new PrintWriter(osw));
-		ctxt.setWriter(writer);
-		return writer;
-	}
+        writer = new ServletWriter(new PrintWriter(osw));
+        ctxt.setWriter(writer);
+        return writer;
+    }
 
     /**
      * Compile the servlet from .java file to .class file
@@ -395,8 +420,8 @@ public abstract class Compiler {
     /**
      * Determine if a compilation is necessary by checking the time stamp of the
      * JSP page with that of the corresponding .class or .java file. If the page
-     * has dependencies, the check is also extended to its dependeants, and so
-     * on. This method can by overidden by a subclasses of Compiler.
+     * has dependencies, the check is also extended to its dependents, and so
+     * on. This method can by overridden by a subclasses of Compiler.
      * 
      * @param checkClass
      *            If true, check against .class file, if false, check against
@@ -413,9 +438,8 @@ public abstract class Compiler {
                     + (ctxt.getOptions().getModificationTestInterval() * 1000) > System
                     .currentTimeMillis()) {
                 return false;
-            } else {
-                jsw.setLastModificationTest(System.currentTimeMillis());
             }
+            jsw.setLastModificationTest(System.currentTimeMillis());
         }
 
         long jspRealLastModified = 0;
@@ -434,7 +458,8 @@ public abstract class Compiler {
             }
             uc.getInputStream().close();
         } catch (Exception e) {
-            log.debug("Problem accessing resource. Treat as outdated.", e);
+            if (log.isDebugEnabled())
+                log.debug("Problem accessing resource. Treat as outdated.", e);
             return true;
         }
 
@@ -469,14 +494,14 @@ public abstract class Compiler {
             return false;
         }
 
-        List depends = jsw.getDependants();
+        List<String> depends = jsw.getDependants();
         if (depends == null) {
             return false;
         }
 
-        Iterator it = depends.iterator();
+        Iterator<String> it = depends.iterator();
         while (it.hasNext()) {
-            String include = (String) it.next();
+            String include = it.next();
             try {
                 URL includeUrl = ctxt.getResource(include);
                 if (includeUrl == null) {
@@ -497,7 +522,9 @@ public abstract class Compiler {
                     return true;
                 }
             } catch (Exception e) {
-                log.debug("Problem accessing resource. Treat as outdated.", e);
+                if (log.isDebugEnabled())
+                    log.debug("Problem accessing resource. Treat as outdated.",
+                            e);
                 return true;
             }
         }
@@ -528,27 +555,26 @@ public abstract class Compiler {
      * Remove generated files
      */
     public void removeGeneratedFiles() {
-        try {
-            String classFileName = ctxt.getClassFileName();
-            if (classFileName != null) {
-                File classFile = new File(classFileName);
-                if (log.isDebugEnabled())
-                    log.debug("Deleting " + classFile);
-                classFile.delete();
-            }
-        } catch (Exception e) {
-            // Remove as much as possible, ignore possible exceptions
-        }
+        removeGeneratedClassFiles();
+
         try {
             String javaFileName = ctxt.getServletJavaFileName();
             if (javaFileName != null) {
                 File javaFile = new File(javaFileName);
                 if (log.isDebugEnabled())
                     log.debug("Deleting " + javaFile);
-                javaFile.delete();
+                if (javaFile.exists()) {
+                    if (!javaFile.delete()) {
+                        log.warn(Localizer.getMessage(
+                                "jsp.warning.compiler.javafile.delete.fail",
+                                javaFile.getAbsolutePath()));
+                    }
+                }
             }
         } catch (Exception e) {
-            // Remove as much as possible, ignore possible exceptions
+            // Remove as much as possible, log possible exceptions
+            log.warn(Localizer.getMessage("jsp.warning.compiler.classfile.delete.fail.unknown"),
+                     e);
         }
     }
 
@@ -559,10 +585,18 @@ public abstract class Compiler {
                 File classFile = new File(classFileName);
                 if (log.isDebugEnabled())
                     log.debug("Deleting " + classFile);
-                classFile.delete();
+                if (classFile.exists()) {
+                    if (!classFile.delete()) {
+                        log.warn(Localizer.getMessage(
+                                "jsp.warning.compiler.classfile.delete.fail",
+                                classFile.getAbsolutePath()));
+                    }
+                }
             }
         } catch (Exception e) {
-            // Remove as much as possible, ignore possible exceptions
+            // Remove as much as possible, log possible exceptions
+            log.warn(Localizer.getMessage("jsp.warning.compiler.classfile.delete.fail.unknown"),
+                     e);
         }
     }
 }
