@@ -35,6 +35,8 @@ import java.util.Map;
 import java.util.StringTokenizer;
 
 import org.apache.jasper.JasperException;
+import org.apache.juli.logging.Log;
+import org.apache.juli.logging.LogFactory;
 import org.eclipse.jdt.core.compiler.IProblem;
 import org.eclipse.jdt.internal.compiler.ClassFile;
 import org.eclipse.jdt.internal.compiler.CompilationResult;
@@ -60,10 +62,12 @@ import org.eclipse.jdt.internal.compiler.problem.DefaultProblemFactory;
  */
 public class JDTCompiler extends org.apache.jasper.compiler.Compiler {
 
-    
+    private final Log log = LogFactory.getLog(JDTCompiler.class); // must not be static
+   
     /** 
      * Compile the servlet from .java file to .class file
      */
+    @Override
     protected void generateClass(String[] smap)
         throws FileNotFoundException, JasperException, Exception {
 
@@ -81,7 +85,8 @@ public class JDTCompiler extends org.apache.jasper.compiler.Compiler {
         final ClassLoader classLoader = ctxt.getJspLoader();
         String[] fileNames = new String[] {sourceFile};
         String[] classNames = new String[] {targetClassName};
-        final ArrayList problemList = new ArrayList();
+        final ArrayList<JavacErrorDetail> problemList =
+            new ArrayList<JavacErrorDetail>();
         
         class CompilationUnit implements ICompilationUnit {
 
@@ -93,42 +98,48 @@ public class JDTCompiler extends org.apache.jasper.compiler.Compiler {
                 this.sourceFile = sourceFile;
             }
 
+            @Override
             public char[] getFileName() {
                 return sourceFile.toCharArray();
             }
             
+            @Override
             public char[] getContents() {
                 char[] result = null;
                 FileInputStream is = null;
+                Reader reader = null;
                 try {
                     is = new FileInputStream(sourceFile);
-                    Reader reader = 
-                        new BufferedReader(new InputStreamReader(is, ctxt.getOptions().getJavaEncoding()));
-                    if (reader != null) {
-                        char[] chars = new char[8192];
-                        StringBuffer buf = new StringBuffer();
-                        int count;
-                        while ((count = reader.read(chars, 0, 
-                                                    chars.length)) > 0) {
-                            buf.append(chars, 0, count);
-                        }
-                        result = new char[buf.length()];
-                        buf.getChars(0, result.length, result, 0);
+                     
+                    reader = new BufferedReader(new InputStreamReader(is,
+                            ctxt.getOptions().getJavaEncoding()));
+                    char[] chars = new char[8192];
+                    StringBuilder buf = new StringBuilder();
+                    int count;
+                    while ((count = reader.read(chars, 0, 
+                                                chars.length)) > 0) {
+                        buf.append(chars, 0, count);
                     }
+                    result = new char[buf.length()];
+                    buf.getChars(0, result.length, result, 0);
                 } catch (IOException e) {
                     log.error("Compilation error", e);
                 } finally {
+                    if (reader != null) {
+                        try {
+                            reader.close();
+                        } catch (IOException ioe) {/*Ignore*/}
+                    }
                     if (is != null) {
                         try {
                             is.close();
-                        } catch (IOException exc) {
-                            // Ignore
-                        }
+                        } catch (IOException exc) {/*Ignore*/}
                     }
                 }
                 return result;
             }
             
+            @Override
             public char[] getMainTypeName() {
                 int dot = className.lastIndexOf('.');
                 if (dot > 0) {
@@ -137,6 +148,7 @@ public class JDTCompiler extends org.apache.jasper.compiler.Compiler {
                 return className.toCharArray();
             }
             
+            @Override
             public char[][] getPackageName() {
                 StringTokenizer izer = 
                     new StringTokenizer(className, ".");
@@ -151,6 +163,7 @@ public class JDTCompiler extends org.apache.jasper.compiler.Compiler {
 
         final INameEnvironment env = new INameEnvironment() {
 
+                @Override
                 public NameEnvironmentAnswer 
                     findType(char[][] compoundTypeName) {
                     String result = "";
@@ -163,6 +176,7 @@ public class JDTCompiler extends org.apache.jasper.compiler.Compiler {
                     return findType(result);
                 }
 
+                @Override
                 public NameEnvironmentAnswer 
                     findType(char[] typeName, 
                              char[][] packageName) {
@@ -235,6 +249,7 @@ public class JDTCompiler extends org.apache.jasper.compiler.Compiler {
                     return is == null;
                 }
 
+                @Override
                 public boolean isPackage(char[][] parentPackageName, 
                                          char[] packageName) {
                     String result = "";
@@ -258,6 +273,7 @@ public class JDTCompiler extends org.apache.jasper.compiler.Compiler {
                     return isPackage(result);
                 }
 
+                @Override
                 public void cleanup() {
                 }
 
@@ -266,7 +282,7 @@ public class JDTCompiler extends org.apache.jasper.compiler.Compiler {
         final IErrorHandlingPolicy policy = 
             DefaultErrorHandlingPolicies.proceedWithAllProblems();
 
-        final Map settings = new HashMap();
+        final Map<String,String> settings = new HashMap<String,String>();
         settings.put(CompilerOptions.OPTION_LineNumberAttribute,
                      CompilerOptions.GENERATE);
         settings.put(CompilerOptions.OPTION_SourceFileAttribute,
@@ -364,6 +380,7 @@ public class JDTCompiler extends org.apache.jasper.compiler.Compiler {
             new DefaultProblemFactory(Locale.getDefault());
         
         final ICompilerRequestor requestor = new ICompilerRequestor() {
+                @Override
                 public void acceptResult(CompilationResult result) {
                     try {
                         if (result.hasProblems()) {
@@ -375,7 +392,7 @@ public class JDTCompiler extends org.apache.jasper.compiler.Compiler {
                                         new String(problems[i].getOriginatingFileName());
                                     try {
                                         problemList.add(ErrorDispatcher.createJavacError
-                                                (name, pageNodes, new StringBuffer(problem.getMessage()), 
+                                                (name, pageNodes, new StringBuilder(problem.getMessage()), 
                                                         problem.getSourceLineNumber(), ctxt));
                                     } catch (JasperException e) {
                                         log.error("Error visiting node", e);
@@ -420,12 +437,13 @@ public class JDTCompiler extends org.apache.jasper.compiler.Compiler {
             String className = classNames[i];
             compilationUnits[i] = new CompilationUnit(fileNames[i], className);
         }
+        CompilerOptions cOptions = new CompilerOptions(settings);
+        cOptions.parseLiteralExpressionsAsConstants = true;
         Compiler compiler = new Compiler(env,
                                          policy,
-                                         settings,
+                                         cOptions,
                                          requestor,
-                                         problemFactory,
-                                         true);
+                                         problemFactory);
         compiler.compile(compilationUnits);
 
         if (!ctxt.keepGenerated()) {
@@ -435,7 +453,7 @@ public class JDTCompiler extends org.apache.jasper.compiler.Compiler {
     
         if (!problemList.isEmpty()) {
             JavacErrorDetail[] jeds = 
-                (JavacErrorDetail[]) problemList.toArray(new JavacErrorDetail[0]);
+                problemList.toArray(new JavacErrorDetail[0]);
             errDispatcher.javacError(jeds);
         }
         
